#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# URLShortener
#------------------------------------------------------------------------------
translations.define("URLShortener",
    ca = u"Servei d'escurçat d'URLs",
    es = u"Servicio de compactación de URLs",
    en = u"URL shortening service"
)

translations.define("URLShortener-plural",
    ca = u"Serveis d'escurçat d'URLs",
    es = u"Servicios de compactación de URLs",
    en = u"URL shortening services"
)

translations.define("URLShortener.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

# BitlyURLShortener
#------------------------------------------------------------------------------
translations.define("BitlyURLShortener",
    ca = u"Compte de bit.ly",
    es = u"Cuenta de bit.ly",
    en = u"bit.ly account"
)

translations.define("BitlyURLShortener-plural",
    ca = u"Comptes de bit.ly",
    es = u"Cuentas de bit.ly",
    en = u"bit.ly accounts"
)

translations.define("BitlyURLShortener.login",
    ca = u"Identificador del compte",
    es = u"Identificador de la cuenta",
    en = u"Login"
)

translations.define("BitlyURLShortener.api_key",
    ca = u"Clau de l'API",
    es = u"Clave de la API",
    en = u"API account"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail.html import Element


class Image(Element):

    tag = "img"
    image = None
    image_factory = "default"
    host = None
    styled_class = False
    accessible_check = False

    def _ready(self):

        if self.image is None \
        or (self.accessible_check and not self.image.is_accessible()):
            self.visible = False
        else:
            self["alt"] = translations(self.image)
            self["src"] = self.image.get_image_uri(
                image_factory = self.image_factory or "default",
                host = self.host
            )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import sys
from threading import Condition
from PyQt4.QtCore import QObject, QUrl, SIGNAL
from PyQt4.QtGui import QApplication, QPrinter
from PyQt4.QtNetwork import QNetworkRequest
from PyQt4.QtWebKit import QWebView

def renderpdf(url, dest):

    app = QApplication(sys.argv)
    web = QWebView()

    request = QNetworkRequest(QUrl(url))
    request.setRawHeader("X-Rendering-PDF", "true")
    web.load(request)

    printer = QPrinter()
    printer.setPageSize(QPrinter.A4)
    printer.setOutputFormat(QPrinter.PdfFormat)
    printer.setOutputFileName(dest)

    def print_file():
        web.print_(printer)
        QApplication.exit()

    QObject.connect(web, SIGNAL("loadFinished(bool)"), print_file)
    app.exec_()

def main():

    if len(sys.argv) != 3:
        sys.stderr.write("Usage: %s URL DEST\n" % sys.argv[0])
        sys.exit(1)

    url = sys.argv[1]
    dest = sys.argv[2]
    renderpdf(url, dest)

if __name__ == "__main__":
    main()




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import cherrypy
from cocktail.translations import (
    get_language,
    set_language
)
from cocktail.controllers import try_decode
from cocktail.controllers.parameters import set_cookie_expiration
from woost.models import Site, Language
from woost.controllers.module import Module


class LanguageModule(Module):

    cookie_duration = 60 * 60 * 24 * 15 # 15 days

    def __init__(self, *args, **kwargs):
        Module.__init__(self, *args, **kwargs)

    def process_request(self, path):

        language = path.pop(0) if path and path[0] in Language.codes else None
        cherrypy.request.language_specified = (language is not None)

        if language is None:
            language = get_language() or self.infer_language()

        cherrypy.response.cookie["language"] = language
        cookie = cherrypy.response.cookie["language"]
        cookie["path"] = "/"
        set_cookie_expiration(cookie, seconds = self.cookie_duration)

        set_language(language)

    def infer_language(self):

        # Check for a language preference in a cookie
        cookie = cherrypy.request.cookie.get("language")

        if cookie:
            return cookie.value

        # Check the 'Accept-Language' header sent by the client
        if Site.main.heed_client_language:
            accept_language = cherrypy.request.headers.get("Accept-Language", None)

            if accept_language:
                available_languages = Language.codes
                best_language = None
                best_score = None

                for chunk in accept_language.split(","):
                    chunk = chunk.strip()
                    score = 1.0
                    chunk_parts = chunk.split(";")

                    if len(chunk_parts) > 1:
                        language = chunk_parts[0]
                        for part in chunk_parts[1:]:
                            if part.startswith("q="):
                                try:
                                    score = float(part[2:])
                                except TypeError:
                                    pass
                                else:
                                    break
                    else:
                        language = chunk

                    if (
                        score
                        and language in available_languages
                        and (best_language is None or score > best_score)
                    ):
                        best_language = language
                        best_score = score

                if best_language:
                    return best_language

        # Fall back to the site's default language
        return Site.main.default_language

    def translate_uri(self, path = None, language = None):

        qs = u""

        if path is None:
            path = cherrypy.request.path_info
            qs = cherrypy.request.query_string

        if isinstance(path, str):
            path = try_decode(path)

        if isinstance(qs, str):
            qs = try_decode(qs)

        if language is None:
            language = get_language()

        path_components = path.strip("/").split("/")
        if path_components and path_components[0] in Language.codes:
            path_components.pop(0)

        path_components.insert(0, language)
        return u"/" + u"/".join(path_components) + (u"?" + qs if qs else u"")




#-*- coding: utf-8 -*-
u"""Provides persistent models that describe schema types.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from weakref import WeakKeyDictionary
from warnings import warn
from cocktail.events import event_handler
from cocktail.pkgutils import import_module, import_object
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore
from woost.models import Item


class UserMember(Item):

    class __metaclass__(Item.__metaclass__):
        def __init__(cls, name, bases, members):
            Item.__metaclass__.__init__(cls, name, bases, members)

            # Automatically create polymorphic members
            if cls.member_class is not schema.Member \
            and not issubclass(
                cls.member_class,
                (schema.RelationMember, schema.Schema)
            ):
                # Make sure the strings for the extension have been loaded
                from woost.extensions.usermodels import strings

                # Default
                if not cls.get_member("member_default"):
                    cls.add_member(cls.create_member_default_member())
                    translations.copy_key(
                        "UserMember.member_default",
                        name + ".member_default",
                        overwrite = False
                    )

                # Enumeration
                if not cls.get_member("member_enumeration"):
                    cls.add_member(cls.create_member_enumeration_member())
                    translations.copy_key(
                        "UserMember.member_enumeration",
                        name + ".member_enumeration",
                        overwrite = False
                    )

    instantiable = False
    visible_from_root = False
    member_class = schema.Member
    member_property_prefix = "member_"
    edit_controls = None
    search_controls = None
    edit_node_class = \
        "woost.extensions.usermodels.usermembereditnode.UserMemberEditNode"
    edit_form = "woost.extensions.usermodels.UserMemberForm"

    groups_order = [
        "description",
        "definition",
        "constraints",
        "behavior"
    ] + (Item.groups_order or [])

    members_order = [
        "label",
        "explanation",
        "member_name",
        "member_translated",
        "member_versioned",
        "member_descriptive",
        "member_indexed",
        "member_unique",
        "member_required",
        "member_member_group",
        "member_listed_by_default",
        "member_editable",
        "member_edit_control",
        "member_searchable",
        "member_search_control",
        "initialization"
    ]

    label = schema.String(
        required = True,
        translated = True,
        descriptive = True,
        member_group = "description"
    )

    explanation = schema.String(
        translated = True,
        edit_control = "woost.views.RichTextEditor",
        listed_by_default = False,
        member_group = "description"
    )

    parent_schema = schema.Reference(
        type = "woost.extensions.usermodels.usermembers.UserModel",
        bidirectional = True,
        visible = False
    )

    parent_collection = schema.Reference(
        type = "woost.extensions.usermodels.usermembers.UserCollection",
        bidirectional = True,
        visible = False
    )

    member_name = schema.String(
        required = True,
        format = r"^[a-zA-Z][a-zA-Z0-9_]*$",
        listed_by_default = False,
        text_search = False,
        member_group = "definition"
    )

    member_translated = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "definition"
    )

    member_versioned = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "definition"
    )

    member_descriptive = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "definition"
    )

    member_indexed = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "definition"
    )

    initialization = schema.CodeBlock(
        language = "python",
        member_group = "code"
    )

    member_unique = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "constraints"
    )

    member_required = schema.Boolean(
        default = False,
        required = True,
        listed_by_default = False,
        member_group = "constraints"
    )

    member_member_group = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "behavior"
    )

    member_listed_by_default = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "behavior"
    )

    member_editable = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "behavior"
    )

    member_edit_control = schema.String(
        listed_by_default = False,
        member_group = "behavior",
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            translations(
                "woost.extensions.usermodels.auto-control",
                language
            )
            if not value else translations(value)
    )

    member_searchable = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "behavior"
    )

    member_search_control = schema.String(
        listed_by_default = False,
        member_group = "behavior",
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            translations(
                "woost.extensions.usermodels.auto-control",
                language
            )
            if not value else translations(value)
    )

    @classmethod
    def create_member_default_member(cls):
        return cls.member_class("member_default",
            member_group = "definition",
        )

    @classmethod
    def create_member_enumeration_member(cls):
        return schema.Collection("member_enumeration",
            required = False,
            items = cls.member_class(required = True),
            member_group = "constraints",
            edit_control = "cocktail.html.TextArea"
        )

    def update_member_definition(self):
        """Creates or updates the schema member described by the item."""
        return self.produce_member(self.get_produced_member())

    def produce_member(self, member = None):

        # Instantiate a new member
        if member is None:
            member = self.member_class()

        # Define translations
        if self.parent_schema:
            trans_key = self.parent_schema.member_name + "." + self.member_name
            translations.clear_key(trans_key)
            for lang in self.translations:
                translations[lang][trans_key] = self.get("label", lang)
                explanation = self.get("explanation", lang)
                if explanation:
                    translations[lang][trans_key + "-explanation"] = \
                        explanation

        # Copy meta-properties, but remove the element from its schema first,
        # to allow it to be renamed
        try:
            parent_schema = member.schema
            if parent_schema:
                prev_schema_order = parent_schema.members_order
                parent_schema.remove_member(member)
            for key in self.__class__.members():
                if key.startswith(self.member_property_prefix):
                    property_name = key[len(self.member_property_prefix):]
                    property_value = self.get(key)
                    setattr(member, property_name, property_value)
        finally:
            if parent_schema:
                parent_schema.add_member(member)
                parent_schema.members_order = prev_schema_order

        # Apply user defined logic to the member
        if self.initialization:
            exec self.initialization in {
                "member": member,
                "user_member": self
            }

        return member

    def get_produced_member(self):
        """Returns a reference to the member produced and registered into the
        application by an earlier call to `produce_member`.
        """
        if self.parent_schema is not None:
            parent_member = self.parent_schema.get_produced_member()
            if parent_member:
                return parent_member.get_member(self.member_name)

        elif self.parent_collection is not None:
            parent_member = self.parent_collection.get_produced_member()
            if parent_member:
                return parent_member.items

    def remove_produced_member(self):
        """Removes the member produced and registered by an earlier call to
        `produce_member`.
        """
        member = self.get_produced_member()

        if member is None:
            return

        if self.parent_schema:
            parent_schema = self.parent_schema.get_produced_member()
            if parent_schema is not None:
                parent_schema.remove_member(self)
        elif self.parent_collection:
            parent_collection = self.parent_collection.get_produced_member()
            if parent_collection:
                parent_collection.items = None

    _v_after_commit_actions = None

    def _after_commit(self, action, *args):

        if self._v_after_commit_actions is None:
            self._v_after_commit_actions = WeakKeyDictionary()

        trans = datastore.connection.transaction_manager.get()
        transaction_actions = self._v_after_commit_actions.get(trans)

        if transaction_actions is None:
            self._v_after_commit_actions[trans] = transaction_actions = set()

        if action not in transaction_actions:
            transaction_actions.add(action)
            def commit_hook(successful, *args):
                if successful:
                    try:
                        action(*args)
                    except Exception, ex:
                        warn(str(ex))
            trans.addAfterCommitHook(commit_hook, args)

    @event_handler
    def handle_changed(cls, e):
        user_member = e.source
        if user_member.is_inserted and user_member.parent_schema:
            user_member._after_commit(user_member.update_member_definition)

    @event_handler
    def handle_deleting(cls, e):
        user_member = e.source
        user_member._after_commit(user_member.remove_produced_member)


class UserBoolean(UserMember):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.Boolean
    edit_controls = [
        "cocktail.html.CheckBox",
        "cocktail.html.RadioSelector"
    ]
    search_controls = [
        "cocktail.html.CheckBox",
        "cocktail.html.RadioSelector"
    ]
    default_member_required = True
    default_member_default = False


class UserInteger(UserMember):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.Integer

    member_min = schema.Integer(
        listed_by_default = False,
        member_group = "constraints"
    )

    member_max = schema.Integer(
        min = member_min,
        listed_by_default = False,
        member_group = "constraints"
    )


class UserDecimal(UserMember):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.Decimal

    member_min = schema.Decimal(
        listed_by_default = False,
        member_group = "constraints"
    )

    member_max = schema.Decimal(
        min = member_min,
        listed_by_default = False,
        member_group = "constraints"
    )


class UserFloat(UserMember):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.Float

    member_min = schema.Float(
        listed_by_default = False,
        member_group = "constraints"
    )

    member_max = schema.Float(
        min = member_min,
        listed_by_default = False,
        member_group = "constraints"
    )


class UserFraction(UserMember):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.Fraction

    member_min = schema.Fraction(
        listed_by_default = False,
        member_group = "constraints"
    )

    member_max = schema.Fraction(
        min = member_min,
        listed_by_default = False,
        member_group = "constraints"
    )


class UserString(UserMember):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.String
    edit_controls = [
        "cocktail.html.TextBox",
        "cocktail.html.PasswordBox",
        "cocktail.html.TextArea",
        "woost.views.RichTextEditor"
    ]
    search_controls = [
        "cocktail.html.TextBox"
    ]

    member_min = schema.Integer(
        listed_by_default = False,
        member_group = "constraints"
    )

    member_max = schema.Integer(
        min = member_min,
        listed_by_default = False,
        member_group = "constraints"
    )

    member_format = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "constraints"
    )

    member_text_search = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "behavior"
    )

    member_normalized_index = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "definition"
    )


class UserDate(UserMember):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.Date
    edit_controls = [
        "cocktail.html.TextBox",
        "cocktail.html.DatePicker"
    ]
    search_controls = [
        "cocktail.html.TextBox",
        "cocktail.html.DatePicker"
    ]

    member_min = schema.Integer(
        listed_by_default = False,
        member_group = "constraints"
    )

    member_max = schema.Integer(
        min = member_min,
        listed_by_default = False,
        member_group = "constraints"
    )


class UserTime(UserMember):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.Time
    edit_controls = [
        "cocktail.html.TextBox",
        "cocktail.html.DatePicker"
    ]
    search_controls = [
        "cocktail.html.TextBox",
        "cocktail.html.DatePicker"
    ]

    member_min = schema.Integer(
        listed_by_default = False,
        member_group = "constraints"
    )

    member_max = schema.Integer(
        min = member_min,
        listed_by_default = False,
        member_group = "constraints"
    )


class UserDateTime(UserMember):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.DateTime
    edit_controls = [
        "cocktail.html.TextBox",
        "cocktail.html.DatePicker"
    ]
    search_controls = [
        "cocktail.html.TextBox",
        "cocktail.html.DatePicker"
    ]

    member_min = schema.Integer(
        listed_by_default = False,
        member_group = "constraints"
    )

    member_max = schema.Integer(
        min = member_min,
        listed_by_default = False,
        member_group = "constraints"
    )


class UserRelation(UserMember):

    instantiable = False
    groups_order = UserMember.groups_order
    member_class = schema.RelationMember

    member_related_key = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "definition"
    )

    member_bidirectional = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "definition"
    )

    member_integral = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "definition"
    )

    member_cascade_delete = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "definition"
    )

    member_text_search = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "behavior"
    )


class UserReference(UserRelation):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.Reference
    edit_controls = [
        "cocktail.html.DropdownSelector",
        "cocktail.html.RadioSelector",
        "woost.views.ItemSelector"
    ]
    search_controls = [
        "cocktail.html.DropdownSelector",
        "cocktail.html.RadioSelector",
        "woost.views.ItemSelector"
    ]

    member_type = schema.Reference(
        class_family = Item,
        member_group = "definition"
    )

    member_class_family = schema.Reference(
        class_family = Item,
        member_group = "definition"
    )

    member_type.exclusive = member_class_family.not_()
    member_class_family = member_type.not_()

    member_cycles_allowed = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "definition"
    )


class UserCollection(UserRelation):

    instantiable = True
    groups_order = UserMember.groups_order
    member_class = schema.Collection
    edit_controls = [
        "cocktail.html.TextArea",
        "cocktail.html.CheckList",
        "cocktail.html.MultipleChoiceSelector"
    ]
    search_controls = [
        "cocktail.html.DropdownSelector",
        "cocktail.html.RadioSelector",
        "woost.views.ItemSelector",
        "cocktail.html.CheckList",
        "cocktail.html.MultipleChoiceSelector"
    ]

    member_items = schema.Reference(
        type = UserMember,
        required = True,
        bidirectional = True,
        related_key = "parent_collection",
        integral = True,
        member_group = "constraints"
    )

    member_min = schema.Integer(
        listed_by_default = False,
        member_group = "constraints"
    )

    member_max = schema.Integer(
        min = member_min,
        listed_by_default = False,
        member_group = "constraints"
    )


class UserModel(UserMember):

    def __init__(self, *args, **kwargs):
        UserMember.__init__(self, *args, **kwargs)

    instantiable = True
    groups_order = UserMember.groups_order
    visible_from_root = True
    member_class = schema.Schema

    edit_node_class = \
        "woost.extensions.usermodels.usermodeleditnode.UserModelEditNode"

    default_member_indexed = True

    plural_label = schema.String(
        required = True,
        translated = True,
        listed_by_default = False,
        member_group = "description"
    )

    package_name = schema.String(
        required = True,
        default = "woost.models",
        format = r"[a-zA-Z][a-zA-Z0-9_]*(\.[a-zA-Z][a-zA-Z0-9_]*)*",
        listed_by_default = False,
        text_search = False,
        member_group = "definition"
    )

    base_model = schema.Reference(
        required = True,
        class_family = Item,
        default = Item,
        member_group = "definition"
    )

    child_members = schema.Collection(
        items = schema.Reference(
            type = UserMember,
            related_end = schema.Reference(),
            bidirectional = True,
            integral = True
        ),
        bidirectional = True,
        related_key = "parent_schema",
        integral = True,
        listed_by_default = False,
        member_group = "definition"
    )

    member_instantiable = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "behavior"
    )

    member_visible_from_root = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "behavior"
    )

    def produce_member(self, member = None):

        # Models aren't created like other regular members
        if member is None:
            member = self.base_model.__metaclass__(
                str(self.member_name),
                (self.base_model,),
                {"full_name": self.package_name + "." + self.member_name}
            )

        # Make the model available at the module level, so that its
        # instances can be pickled
        module = import_module(self.package_name)
        setattr(
            module,
            self.member_name,
            member
        )
        member.__module__ = self.package_name

        # Define translations
        translations.clear_key(self.member_name)

        for lang in self.translations:
            translations[lang][self.member_name] = self.get("label", lang)
            translations[lang][self.member_name + "-plural"] = \
                self.get("plural_label", lang)

            explanation = self.get("explanation", lang)
            if explanation:
                translations[lang][self.member_name + "-explanation"] = \
                    explanation

        member.members_order = []

        # Add or update child members
        for child_user_member in self.child_members:
            member.members_order.append(child_user_member.member_name)

            # Look for an existing version of the child member
            child_member = member.get_member(child_user_member.member_name)

            if child_member:

                # Temporarily remove the child from the schema, in case it
                # needs to be renamed (a member name can't be changed while the
                # member is bound to a schema)
                member.remove_member(child_member)

                # Also, if the member's type has changed, discard it
                if not isinstance(
                    child_member,
                    child_user_member.member_class
                ):
                    child_member = None

            child_member = child_user_member.produce_member(child_member)
            member.add_member(child_member)

        return UserMember.produce_member(self, member)

    def get_produced_member(self):

        full_name = self.package_name + "." + self.member_name

        for cls in Item.schema_tree():
            if cls.full_name == full_name:
                return cls

    def remove_produced_member(self):
        model = self.get_produced_member()
        if model is not None:
            model.select().delete_items()
            datastore.commit()
            self.base_model.remove_derived_schema(model)

    @event_handler
    def handle_changed(cls, e):
        user_member = e.source
        user_member._after_commit(user_member.update_member_definition)




#!/usr/bin/env python
#-*- coding: utf-8 -*-
u"""
Launches the site's application server.
"""
from woost.models.extension import load_extensions
import _PROJECT_MODULE_.models
import _PROJECT_MODULE_.views
from _PROJECT_MODULE_.controllers import _PROJECT_NAME_CMSController

def main():
    load_extensions()
    cms = _PROJECT_NAME_CMSController()
    cms.run()

if __name__ == "__main__":
    main()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
import os
from shutil import rmtree
from tempfile import mkdtemp
from woost.tests.models.basetestcase import BaseTestCase


class FileTestCase(BaseTestCase):

    def setUp(self):
        BaseTestCase.setUp(self)
        self.folder = mkdtemp()

    def tearDown(self):
        BaseTestCase.tearDown(self)
        rmtree(self.folder)

    def test_create_file_from_path(self):

        from woost.models import File
        from hashlib import md5
        path = os.path.join(self.folder, "foo_bar-spam.css")
        content = """div p {
    color: red;
    font-weight: bold
}"""
        f = open(path, "w")
        f.write(content)
        f.close()

        file = File.from_path(path, self.folder, languages = ["ca", "es"])
        assert isinstance(file, File)

        assert file.file_name == "foo_bar-spam.css"
        assert file.file_size == len(content)
        assert file.file_hash == md5(content).digest()
        assert file.mime_type == "text/css"
        assert file.get("title", "ca") == "Foo bar spam"
        assert file.get("title", "ca") == "Foo bar spam"

        upload_path = os.path.join(self.folder, str(file.id))
        assert os.path.exists(upload_path)
        assert os.path.exists(path)

        f = open(upload_path, "r")

        try:
            assert f.read() == content
        finally:
            f.close()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
import os
import cherrypy
from cocktail.controllers import serve_file
from woost.models import (
    Item,
    get_current_user,
    RenderPermission
)
from woost.models.rendering.cache import require_rendering, BadRenderingRequest
from woost.models.rendering.formats import formats_by_extension
from woost.controllers.basecmscontroller import BaseCMSController


class ImagesController(BaseCMSController):
    """A controller that produces, caches and serves images representing the
    different kinds of content managed by the CMS.
    """

    def __call__(self, id, processing, *args, **kwargs):

        # Get the requested element or content type
        item = None

        try:
            item_id = int(id)
        except ValueError:
            for cls in Item.schema_tree():
                if cls.full_name == id:
                    item = cls
                    break
        else:
            item = Item.get_instance(item_id)

        # Make sure the selected element exists
        if item is None:
            raise cherrypy.NotFound()

        # Handle legacy image requests (woost < 0.8)
        if args or kwargs or "(" in processing:
            raise cherrypy.HTTPError(410)

        # Parse the given processing string, splitting the image factory from
        # the image format (ie. "home_thumbnail.png" -> ("home_thumbnail", "PNG"))
        parts = processing.rsplit(".", 1)
        parameters = None

        if len(parts) == 2:
            factory_string, ext = parts
            factory_parts = factory_string.split(".", 1)
            if len(factory_parts) == 1:
                factory_name = factory_string
            else:
                factory_name, parameters = factory_parts
            format = formats_by_extension.get(ext)
            if format is None:
                raise cherrypy.HTTPError(
                    400,
                    "Invalid image extension: %s" % ext
                )
        else:
            factory_name = processing
            format = None

        # Deny access to unauthorized elements
        get_current_user().require_permission(
            RenderPermission,
            target = item,
            image_factory = factory_name
        )

        try:
            image_cache_file = require_rendering(
                item,
                factory_name,
                format,
                parameters
            )
        except BadRenderingRequest, ex:
            raise cherrypy.HTTPError(400, ex.message)

        return serve_file(image_cache_file)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail import schema
from cocktail.iteration import first
from cocktail.translations import translations
from cocktail.html.datadisplay import display_factory
from woost.models import Publishable, Document, News, File
from woost.extensions.opengraph.opengraphtype import OpenGraphType
from woost.extensions.opengraph.utils import export_content

File.default_open_graph_enabled = False
File.default_open_graph_type = None

Publishable.members_order += [
    "open_graph_enabled",
    "open_graph_type"
]

Publishable.add_member(
    schema.Boolean("open_graph_enabled",
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "open_graph"
    )
)

Publishable.add_member(
    schema.Reference("open_graph_type",
        type = OpenGraphType,
        required = Publishable["open_graph_enabled"],
        related_end = schema.Collection(
            block_delete = True
        ),
        default = schema.DynamicDefault(
            lambda: OpenGraphType.get_instance(code = "article")
        ),
        indexed = True,
        listed_by_default = False,
        member_group = "open_graph",
        edit_control = display_factory(
            "cocktail.html.DropdownSelector",
            grouping = lambda type: type.category
        )
    )
)

def _get_publishable_properties(self):

    properties = {
        "og:title": translations(self),
        "og:type": self.open_graph_type.code,
        "og:url": self.get_uri(host = ".")
    }

    image = self.get_open_graph_image()
    if image:
        if isinstance(image, Publishable):
            image = image.get_uri(host = ".")
        properties["og:image"] = image

    video = self.get_open_graph_video()
    if video:
        if isinstance(video, Publishable):
            video = video.get_uri(host = ".")
        properties["og:video"] = video

    return properties

Publishable.get_open_graph_properties = _get_publishable_properties

def _get_document_properties(self):

    properties = Publishable.get_open_graph_properties(self)

    if self.description:
        properties["og:description"] = export_content(self.description)

    return properties

Document.get_open_graph_properties = _get_document_properties

def _get_news_properties(self):

    properties = Document.get_open_graph_properties(self)

    if self.summary:
        properties["og:description"] = export_content(self.summary)

    return properties

News.get_open_graph_properties = _get_news_properties

def _get_publishable_open_graph_image(self):
    return None

Publishable.get_open_graph_image = _get_publishable_open_graph_image

def _get_document_open_graph_image(self):
    return first(attachment
                 for attachment in self.attachments
                 if attachment.resource_type == "image"
                 and attachment.is_accessible())

Document.get_open_graph_image = _get_document_open_graph_image

def _get_publishable_open_graph_video(self):
    return None

Publishable.get_open_graph_video = _get_publishable_open_graph_video

def _get_document_open_graph_video(self):
    return first(attachment
                 for attachment in self.attachments
                 if attachment.resource_type == "video"
                 and attachment.is_accessible())

Document.get_open_graph_video = _get_document_open_graph_video

# Metadata for shop products (only if the 'shop' extension is enabled)
from woost.extensions.shop import ShopExtension

if ShopExtension.instance.enabled:
    from woost.extensions.shop.product import Product
    Product.default_open_graph_type = schema.DynamicDefault(
        lambda: OpenGraphType.get_instance(code = "product")
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from simplejson import loads
from urllib import urlopen, quote
from cocktail import schema
from woost.extensions.shorturls.urlshortener import (
    URLShortener,
    URLShortenerServiceError
)


class BitlyURLShortener(URLShortener):

    instantiable = True

    members_order = [
        "login",
        "api_key"
    ]

    login = schema.String(
        required = True
    )

    api_key = schema.String(
        required = True,
        text_search = False,
        listed_by_default = False
    )

    def shorten_url(self, url):
        response = urlopen(
            "http://api.bitly.com/v3/shorten?"
            "login=%s"
            "&apiKey=%s"
            "&longUrl=%s"
            "&format=json"
            % (
                self.login,
                self.api_key,
                quote(url)
            )
        )
        status = response.getcode()
        body = response.read()

        if status < 200 or status > 299:
            raise URLShortenerServiceError(body)

        return loads(body)["data"]["url"]




#-*- coding: utf-8 -*-
u"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
import os
from shutil import move
import cherrypy
from cocktail.modeling import getter, cached_getter
from cocktail.events import event_handler
from cocktail import schema
from cocktail.schema.exceptions import ValidationError
from cocktail.controllers import context, session
from cocktail.controllers.fileupload import FileUpload
from woost import app
from woost.controllers.backoffice.publishableeditnode \
    import PublishableEditNode


class FileEditNode(PublishableEditNode):

    @cached_getter
    def form_adapter(self):
        adapter = PublishableEditNode.form_adapter(self)
        adapter.exclude(["mime_type"])
        adapter.export_rules.add_rule(ExportUploadInfo())
        adapter.import_rules.add_rule(ImportUploadInfo())
        if self.item.resource_type != "image":
            adapter.exclude("image_effects")
        return adapter

    @cached_getter
    def form_schema(self):

        form_schema = PublishableEditNode.form_schema(self)

        # Not all users can read the same members, try to find its best
        # position
        if form_schema.get_member("local_path"):
            member_params = {"after": "local_path"}
        elif form_schema.get_member("image_effects"):
            member_params = {"before": "image_effects"}
        else:
            member_params = {}

        form_schema.add_member(
            FileUpload("upload",
                hash_algorithm = "md5",
                get_file_destination = lambda upload: self.temp_file_path,
                member_group = "content"
            ),
            **member_params
        )

        if form_schema.get_member("local_path") \
        and not (self.item.is_inserted and not self.item.local_path):
            @form_schema.add_validation
            def validate_only_one_file_source(member, validable, ctx):
                if bool(ctx.get_value("upload")) \
                 + bool(ctx.get_value("local_path")) != 1:
                    yield FileRequiredError(member, validable, ctx)
        else:
            form_schema["upload"].required = \
                not (self.item.is_inserted and self.item.file_name)

        return form_schema

    def iter_changes(self, source = None):

        for member, language in PublishableEditNode.iter_changes(self, source):

            # Ignore differences on the upload field if no file has been
            # uploaded
            if member.name == "upload" \
            and schema.get(self.form_data, "upload", None) is None:
                continue

            yield (member, language)

    @getter
    def temp_file_path(self):
        return app.path("upload", "temp", "%s-%s-%s" % (
            self.item.id,
            session.id,
            self.stack.to_param()
        ))

    @event_handler
    def handle_saving(cls, event):

        # Move the uploaded file to its permanent location
        stack_node = event.source
        src = stack_node.temp_file_path

        if "image_effects" in cherrypy.request.params:
            stack_node.item.image_effects = \
                cherrypy.request.params.get("image_effects")

        if os.path.exists(src):

            dest = stack_node.item.file_path

            if os.path.exists(dest):
                os.remove(dest)

            move(src, dest)


class ExportUploadInfo(schema.Rule):

    def adapt_object(self, context):

        file_name = context.get("file_name")

        if file_name and not context.get("local_path", None):
            context.set("upload", {
                "id": None,
                "file_name": file_name,
                "mime_type": context.get("mime_type"),
                "file_size": context.get("file_size"),
                "file_hash": context.get("file_hash")
            })


class ImportUploadInfo(schema.Rule):

    def adapt_object(self, context):

        context.consume("upload")
        upload = context.get("upload", None)

        if upload and not context.get("local_path", None):
            context.set("file_name", schema.get(upload, "file_name"))
            context.set("mime_type", schema.get(upload, "mime_type"))
            context.set("file_size", schema.get(upload, "file_size"))
            context.set("file_hash", schema.get(upload, "file_hash"))


class FileRequiredError(ValidationError):

    @getter
    def invalid_members(self):
        return self.member["upload"], self.member["local_path"]




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
import re
from cocktail.modeling import classgetter
from cocktail.translations import translations
from cocktail import schema
from cocktail.persistence import datastore
from woost.models.item import Item
from woost.models.language import Language
from woost.models.file import File
from woost.models.publicationschemes import PathResolution
from woost.models.caching import CachingPolicy


class Site(Item):

    indexed = True
    instantiable = False

    groups_order = ["language", "meta", "contact", "pages", "system"]

    members_order = [
        "default_language",
        "backoffice_language",
        "heed_client_language",
        "site_name",
        "logo",
        "icon",
        "keywords",
        "description",
        "organization_name",
        "organization_url",
        "address",
        "town",
        "region",
        "postal_code",
        "country",
        "phone_number",
        "fax_number",
        "email",
        "home",
        "login_page",
        "generic_error_page",
        "not_found_error_page",
        "forbidden_error_page",
        "https_policy",
        "https_persistence",
        "smtp_host",
        "smtp_user",
        "smtp_password",
        "publication_schemes",
        "caching_policies",
        "triggers"
    ]

    @classgetter
    def main(cls):
        return cls.get_instance(qname = "woost.main_site")

    default_language = schema.String(
        required = True,
        default = "en",
        enumeration = lambda ctx: Language.codes,
        listed_by_default = False,
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(value, language, **kwargs),
        member_group = "language"
    )

    backoffice_language = schema.String(
        required = True,
        enumeration = ["en", "es", "ca"],
        default = "en",
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(value, language, **kwargs),
        member_group = "language"
    )

    heed_client_language = schema.Boolean(
        required = True,
        default = True,
        listed_by_default = False,
        member_group = "language"
    )

    organization_name = schema.String(
        translated = True,
        listed_by_default = False,
        member_group = "contact"
    )

    organization_url = schema.String(
        listed_by_default = False,
        member_group = "contact"
    )

    address = schema.String(
        edit_control = "cocktail.html.TextArea",
        member_group = "contact",
        listed_by_default = False
    )

    town = schema.String(
        member_group = "contact",
        listed_by_default = False
    )

    region = schema.String(
        member_group = "contact",
        listed_by_default = False
    )

    postal_code = schema.String(
        member_group = "contact",
        listed_by_default = False
    )

    country = schema.String(
        member_group = "contact",
        listed_by_default = False
    )

    phone_number = schema.String(
        member_group = "contact",
        listed_by_default = False
    )

    fax_number = schema.String(
        member_group = "contact",
        listed_by_default = False
    )

    email = schema.String(
        format = "^.+@.+$",
        member_group = "contact",
        listed_by_default = False
    )

    home = schema.Reference(
        type = "woost.models.Publishable",
        required = True,
        listed_by_default = False,
        member_group = "pages"
    )

    login_page = schema.Reference(
        type = "woost.models.Publishable",
        listed_by_default = False,
        member_group = "pages"
    )

    generic_error_page = schema.Reference(
        type = "woost.models.Publishable",
        listed_by_default = False,
        member_group = "pages"
    )

    not_found_error_page = schema.Reference(
        type = "woost.models.Publishable",
        listed_by_default = False,
        member_group = "pages"
    )

    forbidden_error_page = schema.Reference(
        type = "woost.models.Publishable",
        listed_by_default = False,
        member_group = "pages"
    )

    site_name = schema.String(
        translated = True,
        member_group = "meta",
        listed_by_default = False
    )

    icon = schema.Reference(
        type = File,
        relation_constraints = [File.resource_type.equal("image")],
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "meta"
    )

    logo = schema.Reference(
        type = File,
        relation_constraints = [File.resource_type.equal("image")],
        related_end = schema.Collection(),
        listed_by_default = False,
        member_group = "meta"
    )

    keywords = schema.String(
        translated = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea",
        member_group = "meta"
    )

    description = schema.String(
        translated = True,
        listed_by_default = False,
        edit_control = "cocktail.html.TextArea",
        member_group = "meta"
    )

    https_policy = schema.String(
        required = True,
        default = "never",
        enumeration = [
            "always",
            "never",
            "per_page",
        ],
        translate_value = lambda value, language = None, **kwargs:
            "" if not value
               else translations(
                        "Site.https_policy-%s" % value,
                        language,
                        **kwargs
                    ),
        listed_by_default = False,
        member_group = "system"
    )

    https_persistence = schema.Boolean(
        required = True,
        default = False,
        listed_by_default = False,
        member_group = "system"
    )

    smtp_host = schema.String(
        default = "localhost",
        listed_by_default = False,
        text_search = False,
        member_group = "system"
    )

    smtp_user = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "system"
    )

    smtp_password = schema.String(
        listed_by_default = False,
        text_search = False,
        member_group = "system"
    )

    timezone = schema.String(
        required = False,
        format = re.compile(r'^["GMT"|"UTC"|"PST"|"MST"|"CST"|"EST"]{3}$|^[+-]\d{4}$'),
        text_search = False,
        member_group = "system"
    )

    triggers = schema.Collection(
        items = "woost.models.Trigger",
        bidirectional = True,
        integral = True
    )

    publication_schemes = schema.Collection(
        items = "woost.models.PublicationScheme",
        bidirectional = True,
        integral = True,
        min = 1
    )

    caching_policies = schema.Collection(
        items = schema.Reference(type = CachingPolicy),
        bidirectional = True,
        integral = True,
        related_end = schema.Reference()
    )

    def __translate__(self, language, **kwargs):
        return translations(self.__class__.__name__, language, **kwargs)

    def resolve_path(self, path):
        """Determines the publishable item that matches the indicated path.

        This method identifies a matching publishable item by trying each
        publication scheme defined by the site, in order. Once a scheme finds a
        matching item, the search concludes.

        See L{PublicationScheme.resolve_path} for more details on the resolution
        process.

        @param path: The path to evaluate; A list-like object describing a
            a path relative to the application's root.
        @type path: str list

        @return: A structure containing the matching item and its publication
            details. If no matching item can be found, None is returned
            instead.
        @rtype: L{PathResolution}
        """
        if not path:
            return PathResolution(None, self.home)
        else:
            for pubscheme in self.publication_schemes:
                resolution = pubscheme.resolve_path(path)
                if resolution is not None:
                    return resolution

    def get_path(self, publishable, language = None):
        """Determines the canonical path of the indicated item.

        This method queries each publication scheme defined by the site, in
        order. Once a scheme yields a matching path, the search concludes. That
        first match will be considered the item's canonical path.

        See L{PublicationScheme.get_path} for more details on how paths for
        publishable items are determined.

        @param publishable: The item to get the canonical path for.
        @type publishable: L{Publishable<woost.models.publishable.Publishable>}

        @param language: The language to get the path in (some schemes produce
            different canonical paths for the same content in different
            languages).
        @type language: str

        @return: The publication path for the indicated item, relative to the
            application's root. If none of the site's publication schemes can
            produce a suitable path for the item, None is returned instead.
        @rtype: unicode
        """
        # The path to the home page is always the application root
        if publishable is self.home:
            return ""

        for pubscheme in self.publication_schemes:
            path = pubscheme.get_path(publishable, language)
            if path is not None:
                return path




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

translations.define("woost.extensions.webconsole.page.title",
    ca = u"Intèrpret de Python",
    es = u"Intérprete de Python",
    en = u"Python interpreter"
)

translations.define("woost.extensions.webconsole.controller.title",
    ca = u"Controlador d'intèrpret de Python",
    es = u"Controlador de intérprete de Python",
    en = u"Python interpreter controller"
)

translations.define("woost.extensions.webconsole.template.title",
    ca = u"Plantilla d'intèrpret de Python",
    es = u"Plantilla de intérprete de Python",
    en = u"Python interpreter template"
)

# WebConsolePermission
#------------------------------------------------------------------------------
translations.define("WebConsolePermission",
    ca = u"Permís d'execució de Python",
    es = u"Permiso de ejecución de Python",
    en = u"Python execution permission"
)

translations.define("WebConsolePermission-plural",
    ca = u"Permisos d'execució de Python",
    es = u"Permisos de ejecución de Python",
    en = u"Python execution permissions"
)




#-*- coding: utf-8 -*-
u"""

@author:		    Javier Marrero
@contact:		    javier.marrero@whads.com
@organization:		Whads/Accent SL
@since:			    July 2010
"""

import cherrypy
from cocktail.controllers.erroremail import error_email
from woost.controllers.documentcontroller import DocumentController

class ErrorDocumentController(DocumentController):

    def submit(self):

        if cherrypy.config.has_key('tools.error_email.on') and \
            cherrypy.config['tools.error_email.on']:

            error_email(
                sender = cherrypy.config['tools.error_email.sender'],
                receivers = cherrypy.config['tools.error_email.receivers']
            )



#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2008
"""
from cocktail.schema.expressions import Expression
from woost.models.item import Item
from woost.models.usersession import get_current_user


class OwnershipExpression(Expression):
    """An expression that checks that an item is owned by the active user."""

    def eval(self, context, accessor = None):
        return context.get("owner") is get_current_user()

    def resolve_filter(self, query):
        return Item.owner.equal(get_current_user()).resolve_filter(query)




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			March 2010
"""
from cocktail.translations import translations
from cocktail import schema
from cocktail.html import templates
from woost.models.messagestyles import permission_doesnt_match_style
from woost.models.permission import Permission


class ExportationPermission(Permission):
    """Permission to execute a site's exportation to a destination."""

    instantiable = True

    def _destination_edit_control(parent, obj, member):
        display = templates.new("cocktail.html.DropdownSelector")
        display.empty_label = translations(
            "woost.extensions.staticsite any destination"
        )
        return display

    destination = schema.Reference(
        type = "woost.extensions.staticsite.staticsitedestination.StaticSiteDestination",
        related_key = "destination_permissions",
        bidirectional = True,
        edit_control = _destination_edit_control
    )

    del _destination_edit_control

    def match(self, destination, verbose = False):

        if self.destination and destination is not self.destination:
            print permission_doesnt_match_style("destination doesn't match")
            return False

        return Permission.match(self, verbose)




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			September 2010
"""
from woost.controllers.backoffice.useractions import UserAction
from woost.extensions.mailer.mailing import Mailing, MAILING_FINISHED
from woost.extensions.mailer.sendemailpermission import \
    SendEmailPermission


class SendEmailAction(UserAction):
    included = frozenset(["item_buttons"])
    content_type = Mailing

    def is_permitted(self, user, target):
        return target.is_inserted \
        and not target.status == MAILING_FINISHED \
        and user.has_permission(SendEmailPermission)

SendEmailAction("send_email").register()




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.events import when
from woost.models.trigger import (
    ContentTrigger,
    trigger_responses
)
from woost.extensions.ecommerce.ecommerceorder import ECommerceOrder


class IncomingOrderTrigger(ContentTrigger):

    instantiable = True


@when(ECommerceOrder.incoming)
def _trigger_ecommerceorder_completed_responses(event):
    trigger_responses(
        IncomingOrderTrigger,
        target = event.source
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import Extension


translations.define("ShortURLsExtension",
    ca = u"URLs curtes",
    es = u"URLs cortas",
    en = u"Short URLs"
)

translations.define("ShortURLsPublicationExtension-plural",
    ca = u"URLs curtes",
    es = u"URLs cortas",
    en = u"Short URLs"
)


class ShortURLsExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"""Integració amb serveis d'escurçat d'URLs""",
            "ca"
        )
        self.set("description",
            u"""Integración con servicios de compactación de URLs""",
            "es"
        )
        self.set("description",
            u"""Integration with URL shortening services""",
            "en"
        )

    def _load(self):

        from woost.extensions.shorturls import (
            strings,
            urlshortener,
            bitlyurlshortener
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from time import mktime
from cocktail.modeling import abstractmethod


class ContentRenderer(object):

    enabled = True

    @abstractmethod
    def can_render(self, item):
        """Indicates if the renderer is able to render the given item.

        @param item: The item to evaluate.
        @type item: L{Item<woost.models.item.Item>}

        @return: True if the renderer claims to be able to render the item,
            False otherwise.
        @rtype: bool
        """

    @abstractmethod
    def render(self, item, **kwargs):
        """Produces an image for the given item.

        @param item: The item to render.
        @type item: L{Item<woost.models.item.Item>}

        @param kwargs: Additional parameters that can modify the image produced
            by the renderer. These are implementation dependant, and should
            be described by L{rendering_schema}.

        @return: The image for the given item. If a suitable image file
            exists, the method should return a tuple consisting of a string
            pointing to its path and its MIME type. Otherwise, the method
            should craft the image in-memory and return it as an instance of
            the L{Image<Image.Image>} class.
        @rtype: tuple(str, str) or L{Image<Image.Image>}
        """

    def last_change_in_appearence(self, item):
        """Determines the last time an item was modified in a way that may
        alter its rendering.

        This method is used to check wether cached images are still current.

        @param item: The item to evaluate.
        @type item: L{Item<woost.models.item.Item>}

        @return: The timestamp of the last change to the item that could alter
            the resulting image produced by the L{render} method of this
            renderer.
        @rtype: float
        """
        return mktime(item.last_update_time.timetuple())




#-*- coding: utf-8 -*-
u"""Defines the `PasswordChangeController` class.

.. moduleauthor:: Javier Marrero <javier.marrero@whads.com>
"""
import hashlib
from cocktail import schema
from cocktail.schema.exceptions import ValidationError
from cocktail.controllers import (
    request_property,
    FormProcessor,
    Form
)
from woost.models.emailtemplate import EmailTemplate
from woost.models import Site, User, Publishable
from woost.controllers.documentcontroller import DocumentController


class UserIdentifierNotRegisteredError(ValidationError):
    pass


class UserEmailMissingError(ValidationError):
    pass


def generate_confirmation_hash(identifier):
    hash = hashlib.sha1()
    hash.update(identifier)
    hash.update(Site.main.secret_key)
    return hash.hexdigest()


class PasswordChangeController(FormProcessor, DocumentController):

    class_view = "woost.views.PasswordChangeRequestTemplate"

    class RequestPasswordChangeForm(Form):

        @request_property
        def schema(self):

            user_id_member = self.identifier_member.copy()
            user_id_member.name = "user"

            @user_id_member.add_validation
            def validate_user_identifier_exists(member, value, context):
                if value and self.user is None:
                    yield UserIdentifierNotRegisteredError(member, value, context)

            @user_id_member.add_validation
            def validate_user_has_email(member, value, context):
                if self.user is not None and not self.user.email:
                    yield UserEmailMissingError(member, value, context)

            return schema.Schema("PasswordChangeRequestForm", members = [
                user_id_member
            ])

        @request_property
        def identifier_member(self):
            cms = self.controller.context["cms"]
            return cms.authentication.identifier_field

        @request_property
        def identifier(self):
            return self.data["user"]

        @request_property
        def user(self):
            if self.identifier:
                return User.get_instance(**{
                    self.identifier_member.name: self.identifier
                })

        def after_submit(self):
            confirmation_email_template = EmailTemplate.get_instance(
                qname="woost.views.password_change_confirmation_email_template"
            )
            if confirmation_email_template:
                confirmation_email_template.send({
                    "user": self.user,
                    "confirmation_url": self.confirmation_url
                })

        @request_property
        def confirmation_url(self):
            confirmation_page = Publishable.require_instance(
                qname = 'woost.password_change_confirmation_page'
            )
            return confirmation_page.get_uri(
                host = ".",
                parameters = {
                    "user": self.identifier,
                    "hash": generate_confirmation_hash(self.identifier)
                }
            )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
from shutil import rmtree, copy
from cocktail.events import when
from cocktail.persistence import datastore
from woost import app
from woost.models.item import Item
from woost.models.user import User
from woost.models.permission import RenderPermission
from woost.models.rendering.formats import (
    mime_types_by_format,
    extensions_by_format,
    formats_by_extension,
    default_format
)
from woost.models.rendering.factories import (
    image_factories,
    parse_image_factory_parameters
)

debug = False

def _remove_dir_contents(path):

    # os.path.lexists is not supported on windows
    from cocktail.styled import styled
    exists = getattr(os.path, "lexists", os.path.exists)

    if exists(path):
        for item in os.listdir(path):
            item_path = os.path.join(path, item)
            if debug:
                print styled(" " * 4 + item_path, "red")
            try:
                if os.path.isdir(item_path):
                    rmtree(item_path)
                else:
                    os.remove(item_path)
            except OSError, ex:
                if debug:
                    print styled(ex, "red")

def clear_image_cache(item = None):

    if debug:
        from cocktail.styled import styled
        if item is None:
            print styled("Clearing image cache", "red")
        else:
            print styled("Clearing image cache for", "red"),
            print styled(item, "red", style = "bold")

    # Remove the full cache
    if item is None:
        _remove_dir_contents(app.path("image-cache"))
        _remove_dir_contents(app.path("static", "images"))

    # Remove the cache for a single item
    else:
        from cocktail.styled import styled
        _remove_dir_contents(app.path("image-cache", str(item.id)))
        _remove_dir_contents(app.path("static", "images", str(item.id)))

@when(Item.changed)
@when(Item.deleted)
def _clear_image_cache_after_commit(event):
    item = event.source
    if item.is_inserted:
        datastore.unique_after_commit_hook(
            "woost.models.clear_image_cache-%d" % item.id,
            _clear_image_cache_after_commit_callback,
            item
        )

def _clear_image_cache_after_commit_callback(commit_successful, item):
    if commit_successful:
        clear_image_cache(item)

def require_rendering(
    item,
    factory_name = "default",
    format = None,
    parameters = None):

    factory = image_factories.get(factory_name)

    if factory is None:
        raise BadRenderingRequest("Invalid image factory: %s" % factory_name)

    ext = None

    if format is None:
        if not isinstance(item, type):
            ext = getattr(item, "file_extension", None)

            if ext is not None:
                format = formats_by_extension.get(ext.lstrip(".").lower())
                if format is None:
                    ext = None

    elif format not in mime_types_by_format:
        raise BadRenderingRequest("Invalid image format: %s" % format)

    if format is None:
        format = default_format

    if ext is None:
        ext = extensions_by_format[format]

    file_name = factory_name
    if parameters:
        file_name += "." + parameters
    file_name += "." + ext

    item_id = item.full_name if isinstance(item, type) else str(item.id)

    # If the image hasn't been generated yet, do so and store it in the
    # application's image cache
    image_cache_file = app.path("image-cache", item_id, file_name)

    if not os.path.exists(image_cache_file):

        # Generate the file
        values = parse_image_factory_parameters(factory.parameters, parameters)
        image = factory(item, *values)

        # Store the generated image in the image cache
        try:
            os.mkdir(app.path("image-cache", item_id))
        except OSError:
            pass

        if isinstance(image, basestring):
            try:
                os.remove(image_cache_file)
            except OSError:
                pass

            if hasattr(os, "symlink"):
                os.symlink(image, image_cache_file)
            else:
                copy(image, image_cache_file)
        else:
            if format == 'JPEG' and image.mode not in ('RGB', 'RGBA'):
                image = image.convert('RGBA')
            image.save(image_cache_file, format)

    # If the image is accessible to anonymous users, create a link in the
    # application's static content folder (further requests will be served
    # by the web server, no questions asked).
    if hasattr(os, "symlink"):
        static_publication_link = app.path("static", "images", item_id, file_name)

        if not os.path.lexists(static_publication_link):
            anonymous = User.require_instance(qname ="woost.anonymous_user")

            if anonymous.has_permission(
                RenderPermission,
                target = item,
                image_factory = factory_name
            ):
                try:
                    os.mkdir(app.path("static", "images", item_id))
                except OSError:
                    pass
                os.symlink(image_cache_file, static_publication_link)

    return image_cache_file


class BadRenderingRequest(Exception):
    """An exception raised when trying to render a piece of content using
    invalid parameters (ie. an unknown image factory or image format).
    """




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail import schema
from woost.models.publishable import Publishable
from woost.models.controller import Controller


class URI(Publishable):

    instantiable = True

    groups_order = ["content"]

    members_order = [
        "title",
        "uri"
    ]

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(qname = "woost.uri_controller")
    )

    title = schema.String(
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True,
        member_group = "content"
    )

    uri = schema.String(
        required = True,
        indexed = True,
        member_group = "content"
    )





#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from __future__ import with_statement
from types import GeneratorType
from threading import Lock, Event
from time import time, mktime
from hashlib import md5
import cherrypy
from cherrypy.lib import cptools, http
from woost.controllers import BaseCMSController, get_cache_manager

_cache_lock = Lock()
_cache_requests = {}


class PublishableController(BaseCMSController):
    """Base controller for all publishable items (documents, files, etc)."""

    cache_enabled = True

    cached_headers = (
        "Content-Type",
        "Content-Length",
        "Content-Disposition",
        "Content-Encoding",
        "Last-Modified"
    )

    def __call__(self, **kwargs):

        if self.cache_enabled and cherrypy.request.method == "GET":
            content = self._apply_cache(kwargs)
            if content is not None:
                return content

        return self._produce_content(**kwargs)

    def _apply_cache(self, kwargs):

        if "original_publishable" in self.context:
            return

        request = cherrypy.request
        response = cherrypy.response
        publishable = self.context["publishable"]

        caching_context = {
            "request": request,
            "controller": self
        }
        policy = publishable.get_effective_caching_policy(**caching_context)

        if policy is not None and policy.cache_enabled:

            # Find the unique cache identifier for the requested content
            cache_key = repr(policy.get_content_cache_key(
                publishable,
                **caching_context
            ))

            # Find the last time that the requested element (or its related
            # content) was modified
            content_last_update = policy.get_content_last_update(
                publishable,
                **caching_context
            )

            # Client side cache
            timestamp = None

            if content_last_update:
                timestamp = mktime(content_last_update.timetuple())
                etag_hash = md5()
                etag_hash.update(cache_key)
                etag_hash.update(repr(timestamp))
                response.headers["ETag"] = etag_hash.hexdigest()
                cptools.validate_etags()

            # Server side cache
            if policy.server_side_cache:

                cached_response = \
                    self._get_cached_content(cache_key, policy, timestamp)

                if cached_response is None:

                    cache_request = None

                    with _cache_lock:
                        cached_response = \
                            self._get_cached_content(cache_key, policy, timestamp)

                        if cached_response is None:
                            cache_request = _cache_requests.get(cache_key)
                            if cache_request is None:
                                new_request = True
                                cache_request = Event()
                            else:
                                new_request = False

                    if cache_request is not None:
                        if new_request:
                            content = self._produce_cached_content(
                                cache_key,
                                **kwargs
                            )
                            with _cache_lock:
                                try:
                                    del _cache_requests[cache_key]
                                except KeyError:
                                    pass
                            cache_request.set()
                        else:
                            cache_request.wait()

                if cached_response is not None:
                    headers, content = cached_response
                    cherrypy.response.headers.update(headers)

                return content

    def _get_cached_content(self, cache_key, policy, timestamp = None):

        publishable = self.context["publishable"]
        cache = get_cache_manager().get_cache_region(
            'cached_content', 'woost_cache'
        )

        # Look for a cached response for the specified key
        cached_response = None
        cache_hash = md5(cache_key).hexdigest()

        if cache.has_key(cache_hash):
            cache_entry = cache.get(cache_hash)

            if cache_entry:
                entry_creation_time, cached_response, cached_key = cache_entry

                valid_key = (cached_key == cache_key)

                # Validate entry expiration
                expiration = policy.cache_expiration
                expired = (expiration is not None
                           and time() - entry_creation_time > expiration * 60)

                current = timestamp is None or entry_creation_time >= timestamp

                if expired or not current or not valid_key:
                    cache.remove(cache_hash)
                    cached_response = None

        return cached_response

    def _produce_cached_content(self, cache_key, **kwargs):

        cache = get_cache_manager().get_cache_region(
            'cached_content', 'woost_cache'
        )
        content = self._produce_content(**kwargs)

        if isinstance(content, GeneratorType):
            content = "".join(content)

        # Collect headers that should be included in the cache
        headers = {}
        response_headers = cherrypy.response.headers

        for header_name in self.cached_headers:
            header_value = response_headers.get(header_name)
            if header_value:
                headers[header_name] = header_value

        # Store the response in the cache
        entry_creation_time = time()
        cached_response = (headers, content)
        cache_hash = md5(cache_key).hexdigest()
        cache.put(cache_hash, (entry_creation_time, cached_response, cache_key))

        return content

    def _produce_content(self, **kwargs):
        return BaseCMSController.__call__(self, **kwargs)




#-*- coding: utf-8 -*-
"""

@author:		Jordi Fernández
@contact:		jordi.fernandez@whads.com
@organization:	Whads/Accent SL
@since:			June 2009
"""
from cocktail import schema
from cocktail.persistence import datastore
from woost.models.triggerresponse import TriggerResponse
from woost.extensions.workflow.state import State

class SetStateTriggerResponse(TriggerResponse):
    """A trigger response that allows to change an Item state."""
    instantiable = True

    target_state = schema.Reference(
        type = State,
	    required = True,
	    related_end = schema.Collection(cascade_delete = True),
        edit_control = "cocktail.html.DropdownSelector"
    )

    def execute(self, items, user, batch = False, **context):
        for item in items:
            item.workflow_state = self.target_state




#-*- coding: utf-8 -*-
"""

@author:		Javier Marrero
@contact:		javier.marrero@whads.com
@organization:	Whads/Accent SL
@since:			January 2010
"""
from cocktail.translations import translations

translations.define("woost.extensions.signup.signup_controller.title",
    ca = u"Controlador d'alta d'usuaris",
    es = u"Controlador de alta de usuarios",
    en = u"Sign Up controller"
)

translations.define("woost.extensions.signup.signup_confirmation_controller.title",
    ca = u"Controlador de confirmació d'alta d'usuaris",
    es = u"Controlador de confirmación alta de usuarios",
    en = u"Sign Up confirmation controller"
)

translations.define("woost.extensions.signup.signup_template.title",
    ca = u"Plantilla d'alta d'usuaris",
    es = u"Plantilla de alta de usuarios",
    en = u"Sign up template"
)

translations.define(
    "woost.extensions.signup.signup_confirmation_template.title",
    ca = u"Plantilla de confirmació d'alta d'usuaris",
    es = u"Plantilla de confirmación de alta de usuarios",
    en = u"Sign up confirmation template"
)

translations.define(
    "woost.extensions.signup.signup_confirmation_email_template.title",
    ca = u"Plantilla de correu electrònic de confirmació d'alta d'usuari",
    es = u"Plantilla de email de confirmación de alta de usuario",
    en = u"Sign Up Confirmation email template"
)

translations.define(
    "woost.extensions.signup.signup_confirmation_email_template.subject",
    ca = u"Confirmació de compte d'usuari",
    es = u"Confirmación de cuenta de usuario",
    en = u"User account confirmation"
)

translations.define(
    "woost.extensions.signup.signup_confirmation_email_template.body",
    ca = u"Hola ${user.email}:<br/><br/>"
         u"Fes clic per confirmar el teu compte d'usuari:<br/>"
         u"<a href='${confirmation_url}'>${confirmation_url}</a>",
    es = u"Hola ${user.email}:<br/><br/>"
         u"Haz clic para confirmar tu cuenta de usuario:<br/>"
         u"<a href='${confirmation_url}'>${confirmation_url}</a>",
    en = u"Hello ${user.email},<br/><br/>"
         u"Click here to confirm your email account:<br/>"
         u"<a href='${confirmation_url}'>${confirmation_url}</a>"
)

translations.define(
    "woost.extensions.signup.signup_confirmation_target.title",
    ca = u"Confirmació d'alta d'usuari",
    es = u"Confirmación de alta de usuario",
    en = u"Sign Up Confirmation Page"
)

translations.define(
    "woost.extensions.signup.signup_page.title",
    ca = u"Registre d'usuari",
    es = u"Registro de usuario",
    en = u"Sign Up"
)

# Sign up Page
translations.define("SignUpPage",
    ca = u"Pàgina d'alta d'usuari",
    es = u"Página de alta de usuario",
    en = u"Sign up page"
)

translations.define("SignUpPage-plural",
    ca = u"Pàgines d'alta d'usuari",
    es = u"Páginas de alta de usuario",
    en = u"Sign up pages"
)

translations.define("SignUpPage.signup_process",
    ca = u"Procés d'alta",
    es = u"Proceso de alta",
    en = u"Sign up process"
)

translations.define("SignUpPage.user_type",
    ca = u"Tipus d'usuari",
    es = u"Tipo de usuario",
    en = u"User type"
)

translations.define("SignUpPage.roles",
    ca = u"Rols per defecte",
    es = u"Roles por defecto",
    en = u"Default roles"
)

translations.define("SignUpPage.roles-explanation",
    ca = u"Rols que seran assignats després de l'alta",
    es = u"Roles que serán asigandos tras el alta",
    en = u"Roles that will be assigned after sign up"
)

translations.define("SignUpPage.confirmation_target",
    ca = u"Document de confirmació",
    es = u"Documento de confirmación",
    en = u"Confirmation document"
)

translations.define("SignUpPage.confirmation_target-explanation",
    ca = u"Document on finalitza el procés d'alta i es gestiona la confirmació si correspon",
    es = u"Documento donde finaliza el proceso de alta y se gestiona la confirmación si corresponde",
    en = u"Document where ends the sign up process and will be take place the sign up confirmation process if is required"
)

translations.define("SignUpPage.confirmation_email_template",
    ca = u"Plantilla de correu electrònic de confirmació",
    es = u"Plantilla de correo electrónico de confirmación",
    en = u"Sign up email template"
)

translations.define("SignUpPage.confirmation_email_template-explanation",
    ca = u"""Assignar una plantilla de correu electrònic de confirmació activa
        automàticament el sistema de confirmació per correu electrònic en el
        procés d'alta.""",
    es = u"""Asignar una plantilla de email de confirmación, activa
        automáticamente el sistema de confirmación por email en el proceso de
        alta""",
    en = u"""Selecting a sign up email template will automatically enable the
        email confirmation system in the sign up process"""
)

translations.define("SignUpForm.password_confirmation",
    ca = u"Confirmar la contrasenya",
    es = u"Confirmar la contraseña",
    en = u"Confirm password",
    pt = u"Confirmar Palavra-passe"
)

translations.define("woost.extensions.signup.SignupSuccessfulMessage",
    ca = u"Alta realitzada amb èxit.",
    es = u"Alta realizada con éxito.",
    en = u"Thanks for signing up!",
    pt = u"Obrigado por se inscrever."
)
translations.define("woost.extensions.signup.SignupConfirmationMessageNotice",
    ca = u"""
        En breu rebrà un missatge a la bústia de
        correu electrònic que ens ha facilitat. Segueixi les indicacions del
        correu per tal de confirmar la seva alta a la web. Gràcies.
    """,
    es = u"""
        En breve recibirá un mensaje en el buzón de
        correo electrónico que nos ha indicado. Siga los pasos que allí le
        indicamos para que confirme su alta en la web. Gracias.
    """,
    en = u"""
        We've sent you an email with the necessary
        instructions to complete the registration process.
    """,
    pt = u"""
        Em poucos momentos receberá um mail de
        confirmação do registo.
    """
)

translations.define("woost.extensions.signup.SignupSuccessfulConfirmationMessage",
    ca = u"El procés d'alta d'usuari s'ha realitzat correctament.",
    es = u"El proceso de alta de usuario se ha completado correctamente.",
    en = u"The user registration process has been completed successfully.",
    pt = u"Registro confirmado."
)

translations.define("woost.extensions.signup.SignupFailConfirmationMessage",
    ca = u"S'ha produït un error en el procés de confirmació.",
    es = u"Se ha producido un error en el proceso de confirmación.",
    en = u"An error has occurred in the confirmation process.",
    pt = u"Houve um erro no processo de confirmação."
)

# User
#------------------------------------------------------------------------------
translations.define("User.confirmed_email",
    ca = u"Correu electrònic confirmat",
    es = u"Correo electrónico confirmado",
    en = u"Confirmed email"
)



#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

# TwitterPublicationExtension
#------------------------------------------------------------------------------
translations.define("TwitterPublicationExtension.targets",
    ca = u"Destins",
    es = u"Destinos",
    en = u"Targets"
)

# TweetPermission
#------------------------------------------------------------------------------
translations.define("TweetPermission",
    ca = u"Permís de publicació a Twitter",
    es = u"Permiso de publicación en Twitter",
    en = u"Permission for publishing to Twitter"
)

translations.define("TweetPermission-plural",
    ca = u"Permisos de publicació a Twitter",
    es = u"Permisos de publicación en Twitter",
    en = u"Permissions for publishing to Twitter"
)

translations.define("TweetPermission.targets",
    ca = u"Destins permesos",
    es = u"Destinos permitidos",
    en = u"Allowed targets"
)

# TwitterPublicationTarget
#------------------------------------------------------------------------------
translations.define("TwitterPublicationTarget",
    ca = u"Destí de publicació a Twitter",
    es = u"Destino de publicación en Twitter",
    en = u"Twitter publication target"
)

translations.define("TwitterPublicationTarget",
    ca = u"Destins de publicació a Twitter",
    es = u"Destinos de publicación en Twitter",
    en = u"Twitter publication targets"
)

translations.define("TwitterPublicationTarget.title",
    ca = u"Nom",
    es = u"Nombre",
    en = u"Name"
)

translations.define("TwitterPublicationTarget.account",
    ca = u"Compte de Twitter",
    es = u"Cuenta de Twitter",
    en = u"Twitter account"
)

translations.define("TwitterPublicationTarget.app_id",
    ca = u"Identificador d'aplicació de Twitter",
    es = u"Identificador de aplicación de Twitter",
    en = u"Twitter application id"
)

translations.define("TwitterPublicationTarget.app_secret",
    ca = u"Secret d'aplicació de Twitter",
    es = u"Secreto de aplicación de Twitter",
    en = u"Twitter application secret"
)

translations.define("TwitterPublicationTarget.auth_token",
    ca = u"Autorització",
    es = u"Autorización",
    en = u"Authorization"
)

translations.define("TwitterPublicationTarget.auth_token-conceded",
    ca = u"Concedida",
    es = u"Concedida",
    en = u"Granted"
)

translations.define("TwitterPublicationTarget.auth_token-pending",
    ca = u"Pendent",
    es = u"Pendiente",
    en = u"Pending"
)

translations.define("TwitterPublicationTarget.auth_secret",
    ca = u"Secret d'autorització de Twitter",
    es = u"Secreto de autorización de Twitter",
    en = u"Twitter authorization secret"
)

translations.define("TwitterPublicationTarget.url_shortener",
    ca = u"Escurçador d'URLs",
    es = u"Compactador de URLs",
    en = u"URL shortener"
)

translations.define("TwitterPublicationTarget.languages",
    ca = u"Idiomes a publicar",
    es = u"Idiomas a publicar",
    en = u"Published languages"
)

# twitter_auth user action
#------------------------------------------------------------------------------
translations.define("Action twitter_auth",
    ca = u"Autoritzar",
    es = u"Autorizar",
    en = u"Authorize"
)

translations.define(
    "woost.extensions.twitterpublication.successful_authorization_notice",
    ca = u"S'ha completat l'autorització de forma satisfactòria",
    es = u"Se ha completado la autorización de forma satisfactoria",
    en = u"Authorization granted"
)

# tweet user action
#------------------------------------------------------------------------------
translations.define("Action tweet",
    ca = u"Publicar a Twitter",
    es = u"Publicar en Twitter",
    en = u"Tweet"
)

# TwitterAPIError
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.twitterpublication.exceptions.TwitterAPIError-instance",
    ca = lambda instance: u"Error de comunicació amb Twitter: <pre>%s</pre>"
                          % instance.response,
    es = lambda instance: u"Error de comunicación con Twitter: <pre>%s</pre>"
                          % instance.response,
    en = lambda instance: u"Twitter API error: <pre>%s</pre>"
                          % instance.response
)

# TwitterPublicationForm
#------------------------------------------------------------------------------
translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView.section_title",
    ca = u"Publicar a Twitter",
    es = u"Publicar en Twitter",
    en = u"Publish to Twitter"
)

translations.define("TwitterPublicationForm.subset",
    ca = u"Contingut a publicar",
    es = u"Contenido a publicar",
    en = u"Published items"
)

translations.define("TwitterPublicationForm.published_languages",
    ca = u"Idiomes a publicar",
    es = u"Idiomas a publicar",
    en = u"Published languages"
)

translations.define("TwitterPublicationForm.publication_targets",
    ca = u"Destins on es publicarà el contingut",
    es = u"Destinos donde se publicará el contenido",
    en = u"Publication targets"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "publication_form.submit_button",
    ca = u"Publicar",
    es = u"Publicar",
    en = u"Publish"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "publication_form.check_button",
    ca = u"Comprovar",
    es = u"Comprobar",
    en = u"Check"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "already_published",
    ca = u"Ja publicat",
    es = u"Ya publicado",
    en = u"Already published"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "not_yet_published",
    ca = u"Encara no s'ha publicat",
    es = u"Todavía no se ha publicado",
    en = u"Not published yet"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "success",
    ca = u"Publicat amb èxit",
    es = u"Publicado con éxito",
    en = u"Published successfully"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "publishable_header",
    ca = u"Element",
    es = u"Elemento",
    en = u"Item"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "target_header",
    ca = u"Destí",
    es = u"Destino",
    en = u"Destination"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "language_header",
    ca = u"Idioma",
    es = u"Idioma",
    en = u"Language"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "outcome_header",
    ca = u"Resultat",
    es = u"Resultado",
    en = u"Result"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "publish_again_link",
    ca = u"Tornar a publicar",
    es = u"Volver a publicar",
    en = u"Publish again"
)

translations.define(
    "woost.extensions.twitterpublication.TwitterPublicationView."
    "loading_sign",
    ca = u"Enviant la petició a Twitter...",
    es = u"Enviando la petición a Twitter...",
    en = u"Sending the request to Twitter..."
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import os
import Image
from tempfile import mkdtemp
from shutil import rmtree
from subprocess import Popen
from woost.models.publishable import Publishable
from woost.models.user import User
from woost.models.rendering.contentrenderer import ContentRenderer
from woost.models.rendering.contentrenderersregistry import content_renderers


class HTMLRenderer(ContentRenderer):
    """A content renderer that handles XHTML/HTML pages."""

    enabled = False

    mime_types = set([
        "text/html",
        "text/xhtml",
        "application/xhtml"
    ])

    def can_render(self, item):
        return (
            isinstance(item, Publishable)
            and item.mime_type in self.mime_types
            and item.is_accessible(
                user = User.get_instance(qname = "woost.anonymous_user")
            )
        )

    def render(self, item, window_width = None, window_height = None):

        temp_path = mkdtemp()

        try:
            temp_image_file = os.path.join(temp_path, "thumbnail.png")

            command = "python -m woost.models.rendering.renderurl %s %s" \
                % (item.get_uri(host = "."), temp_image_file)

            if window_width is not None:
                command += " --min-width %d" % window_width

            if window_height is not None:
                command += " --min-width %d" % window_height

            Popen(command, shell = True).wait()
            return Image.open(temp_image_file)

        finally:
            rmtree(temp_path)


try:
    import PyQt4.QtWebKit
except ImportError:
    pass
else:
    content_renderers.register(HTMLRenderer())




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2009
"""
from cocktail.modeling import classgetter, DictWrapper
import cherrypy
from cocktail.controllers import serve_file
from woost.models.rendering import require_rendering
from woost.controllers.publishablecontroller import PublishableController


class FileController(PublishableController):
    """A controller that serves the files managed by the CMS."""

    def _produce_content(self, disposition = "inline"):
        file = self.context["publishable"]

        if file.image_effects:
            file_path = require_rendering(file)
        else:
            file_path = file.file_path

        return serve_file(
            file_path,
            name = file.file_name,
            content_type = file.mime_type,
            disposition = disposition)




#-*- coding: utf-8 -*-
u"""Defines the `Door` class.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from __future__ import with_statement
from threading import Lock, Event
from contextlib import contextmanager
from cocktail.modeling import getter


class Door(object):
    """An object that enables or disables concurrent access to a shared state.
    """

    _exclusive_access_interval = 0.1

    def __init__(self, lock = None):

        if lock is None:
            lock = Lock()

        self.__lock = lock
        self.__concurrency = 0
        self.__empty = Event()

    def enter(self):
        """Request clearence to begin accessing the shared state."""
        with self.__lock:
            self.__concurrency += 1
            self.__empty.clear()

    def leave(self):
        """Notify the end of access to the shared state."""
        with self.__lock:
            self.__concurrency -= 1
            if not self.__concurrency:
                self.__empty.set()

    def lock(self):
        """Prevent concurrent access to the shared state.

        The method waits until no other thread is accessing the shared state
        (ie. no one's inside) and then prevents further access to it until the
        `unlock` method is called.
        """
        while True:
            self.__empty.wait()
            self.__lock.acquire()
            if not self.__concurrency:
                break
            self.__lock.release()

    def unlock(self):
        """Re-enable concurrent access to the shared state after a call to
        `lock`.
        """
        self.__lock.release()

    # Implement the context manager protocol:

    def __enter__(self):
        self.enter()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.leave()

    @getter
    @contextmanager
    def locking(self):
        """A context manager that locks and unlocks the door."""
        self.lock()
        try:
            yield self
        finally:
            self.unlock()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
import cherrypy
from cocktail import schema
from cocktail.controllers import get_parameter, Location
from woost.controllers.basecmscontroller import BaseCMSController
from woost.extensions.payments.dummypaymentgateway \
    import DummyPaymentGateway
from urllib import urlopen


class DummyPaymentGatewayController(BaseCMSController):
    """A controller used by L{DummyPaymentGateway
    <woost.extensions.payments.dummypaymentgateway.DummyPaymentGateway>}
    instances to simulate transaction payments.
    """

    def __call__(self, **parameters):

        # Get references to the gateway and payment
        gateway = get_parameter(
            schema.Reference("gateway_id", type = DummyPaymentGateway)
        )

        if gateway is None:
            raise ValueError("Wrong payment gateway")

        payment_id = get_parameter(schema.String("payment_id"))
        payment = payment_id and gateway.get_payment(payment_id) or None

        if payment is None:
            raise ValueError("Wrong payment id (%s)" % payment_id)

        # Notify the payment to the application
        cms = self.context["cms"]
        notification_uri = Location.get_current_host()
        notification_uri.path_info = cms.application_uri(
            "payment_notification",
            payment_id = payment_id
        )
        urlopen(str(notification_uri))

        # Redirect the user after the transaction's over
        redirection = None

        if gateway.payment_status == "accepted":
            redirection = gateway.payment_successful_page
        elif gateway.payment_status == "failed":
            redirection = gateway.payment_failed_page

        raise cherrypy.HTTPRedirect(
            (redirection and cms.uri(redirection)
            or cms.application_uri()).encode("utf-8")
        )




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail import schema
from woost.models import Item


class ProductCategory(Item):

    members_order = [
        "title",
        "parent",
        "children",
        "products"
    ]

    title = schema.String(
        required = True,
        indexed = True,
        normalized_index = True,
        full_text_indexed = True,
        descriptive = True,
        translated = True
    )

    parent = schema.Reference(
        type = "woost.extensions.shop.productcategory.ProductCategory",
        bidirectional = True
    )

    children = schema.Collection(
        items = "woost.extensions.shop.productcategory.ProductCategory",
        bidirectional = True
    )

    products = schema.Collection(
        items = "woost.extensions.shop.product.Product",
        bidirectional = True
    )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import Image
import ImageEnhance
import ImageFilter
from cocktail.modeling import OrderedDict
from woost.models.file import File

image_effects = OrderedDict()

def image_effect(func):
    image_effects[func.func_name] = func
    return func

def resolve_px(value, size):
    if isinstance(value, float):
        value = int(size * value)
    if value < 0:
        value = size + value
    return value

def resolve_color(value):

    if isinstance(value, basestring):

        if len(value) in (3, 4):
            value = tuple(int(d * 2, 16) for d in value)
        elif len(value) == 6:
            value = (
                int(value[0:2], 16),
                int(value[2:4], 16),
                int(value[4:6], 16)
            )
        elif len(value) == 8:
            value = (
                int(value[0:2], 16),
                int(value[2:4], 16),
                int(value[4:6], 16),
                int(value[6:8], 16)
            )
        else:
            raise ValueError("Invalid color: " + value)

    return value

@image_effect
def thumbnail(image, width = None, height = None, filter = Image.ANTIALIAS):

    im_width, im_height = image.size

    if width is None:
        width = im_width

    if height is None:
        height = im_height

    image.thumbnail(
        (resolve_px(width, im_width),
         resolve_px(height, im_height)),
        filter
    )
    return image

@image_effect
def crop(image, x1, y1, x2, y2):

    width, height = image.size

    return image.crop((
        resolve_px(x1, width),
        resolve_px(y1, height),
        resolve_px(x2, width),
        resolve_px(y2, height)
    ))

@image_effect
def fill(image, width, height, crop = "center", filter = Image.ANTIALIAS):

    source_width, source_height = image.size
    source_ratio = float(source_width) / source_height
    target_ratio = float(width) / height

    if source_ratio > target_ratio:
        target_width = int(height * source_ratio)
        target_height = height
    else:
        target_width = width
        target_height = int(width * (1 / source_ratio))

    image = image.resize((target_width, target_height), filter)

    if crop:
        if crop == "center":
            offset_x = (target_width - width) / 2
            offset_y = (target_height - height) / 2
            image = image.crop((
                offset_x,
                offset_y,
                width + offset_x,
                height + offset_y
            ))
        elif crop == 'top':
            offset_x = (target_width - width) / 2
            offset_y = 0
            image = image.crop((
                offset_x,
                offset_y,
                width + offset_x,
                height + offset_y
            ))
        else:
            raise ValueError("crop = %s not implemented" % crop)

    return image

@image_effect
def rotate(image, angle, expand = True, filter = Image.BICUBIC):
    return image.rotate(angle, filter, expand)

@image_effect
def color(image, level):
    return ImageEnhance.Color(image).enhance(level)

@image_effect
def brightness(image, level):
    return ImageEnhance.Brightness(image).enhance(level)

@image_effect
def contrast(image, level):
    return ImageEnhance.Contrast(image).enhance(level)

@image_effect
def sharpness(image, level):
    return ImageEnhance.Sharpness(image).enhance(level)

@image_effect
def frame(
    image,
    edge = 1,
    edge_color = (0,0,0),
    padding = 0,
    padding_color = (255,255,255)
):
    edge_color = resolve_color(edge_color)
    padding_color = resolve_color(padding_color)

    # Create the canvas
    width, height = image.size
    offset = edge + padding
    canvas = Image.new("RGBA", (width + offset * 2, height + offset * 2))

    # Paint the border
    if edge:
        canvas.paste(edge_color, None)

    # Paint the padding color
    canvas.paste(
        padding_color,
        (edge,
         edge,
         width + offset * 2 - edge,
         height + offset * 2 - edge)
    )

    # Paste the original image over the frame
    canvas.paste(
        image,
        (offset, offset),
        image if image.mode in ("1", "L", "RGBA") else None
    )

    return canvas

@image_effect
def shadow(
    image,
    offset = 5,
    color = (70,70,70),
    padding = 8,
    iterations = 3):

    # Create the backdrop image -- a box in the background colour with a
    # shadow on it.
    total_width = image.size[0] + abs(offset) + 2 * padding
    total_height = image.size[1] + abs(offset) + 2 * padding
    back = Image.new("RGBA", (total_width, total_height))

    # Place the shadow, taking into account the offset from the image
    shadow_left = padding + max(offset, 0)
    shadow_top = padding + max(offset, 0)
    color = resolve_color(color)
    back.paste(color, [
        shadow_left,
        shadow_top,
        shadow_left + image.size[0],
        shadow_top + image.size[1]
    ])

    # Apply the filter to blur the edges of the shadow.  Since a small kernel
    # is used, the filter must be applied repeatedly to get a decent blur.
    for n in range(iterations):
        back = back.filter(ImageFilter.BLUR)

    # Paste the input image onto the shadow backdrop
    image_left = padding - min(offset, 0)
    image_top = padding - min(offset, 0)
    back.paste(
        image,
        (image_left, image_top),
        image if image.mode in ("1", "L", "RGBA") else None
    )

    return back

@image_effect
def reduce_opacity(image, opacity):
    """An image effect that reduces the opacity of the given image."""

    if opacity < 0 or opacity > 1:
        raise ValueError(
            "The 'opacity' parameter of the 'reduce_opacity' image effect "
            "must be a floating point number between 0 and 1: got %s instead"
            % opacity
        )

    if image.mode != 'RGBA':
        image = image.convert('RGBA')
    else:
        image = image.copy()

    alpha = image.split()[3]
    alpha = ImageEnhance.Brightness(alpha).enhance(opacity)
    image.putalpha(alpha)
    return image

@image_effect
def watermark(image, mark_image, position = "middle", opacity = 1):

    if isinstance(mark_image, File):
        mark_image = Image.open(mark.file_path)

    if opacity < 1:
        mark_image = reduce_opacity(mark_image, opacity)

    if image.mode != 'RGBA':
        image = image.convert('RGBA')

    # Create a transparent layer the size of the image and draw the
    # watermark in that layer.
    layer = Image.new('RGBA', image.size, (0,0,0,0))

    if position == 'tile':
        for y in range(0, image.size[1], mark_image.size[1]):
            for x in range(0, image.size[0], mark_image.size[0]):
                layer.paste(mark_image, (x, y), mark_image)
    elif position == 'scale':
        # scale, but preserve the aspect ratio
        ratio = min(
            float(image.size[0]) / mark_image.size[0],
            float(image.size[1]) / mark_image.size[1]
        )
        w = int(mark_image.size[0] * ratio)
        h = int(mark_image.size[1] * ratio)
        mark_image = mark_image.resize((w, h))
        layer.paste(
            mark_image,
            ((image.size[0] - w) / 2, (image.size[1] - h) / 2),
            mark_image
        )
    elif position == 'middle':
        layer.paste(
            mark_image,
            (
                (image.size[0] - mark_image.size[0]) / 2,
                (image.size[1] - mark_image.size[1]) / 2
            ),
            mark_image
        )
    else:
        raise ValueError(
            "Must specify position parameter [tile,scale,middle]."
        )
    #TODO
    # top-left      top         top-right
    # left          middle      right
    # bottom-left   bottom      bottom-right
    # margin

    # composite the watermark with the layer
    return Image.composite(layer, image, layer)

@image_effect
def flip(image, direction):
    return image.transpose(direction)

@image_effect
def fit(image, width, height, crop = "center", filter = Image.ANTIALIAS):

    if image.size[0] > width or image.size[1] > height:
        if image.size[0] >= image.size[1]:
            image = fill(image, width, height, crop, filter)
        else:
            image.thumbnail((width, height), filter)

    return image

@image_effect
def align(
    image,
    width = None,
    height = None,
    halign = "center",
    valign = "center",
    background = None):

    needs_halign = (width and image.size[0] < width)
    needs_valign = (height and image.size[1] < height)

    if needs_halign or needs_valign:

        if background:
            background = resolve_color(background)
        elif image.mode == "RGBA":
            background = (0,0,0,0)

        copy = Image.new(
            "RGBA" if background and len(background) == 4 else "RGB",
            (
                width or image.size[0],
                height or image.size[1]
            ),
            background
        )

        x = 0
        y = 0

        if needs_halign:
            if halign == "left":
                pass
            elif halign == "center":
                x = width / 2 - image.size[0] / 2
            elif halign == "right":
                x = width - image.size[0]
            else:
                raise ValueError(
                    "Invalid parameter: align(halign = %r)" % halign
                )

        if needs_valign:
            if valign == "top":
                pass
            elif valign == "center":
                y = height / 2 - image.size[1] / 2
            elif valign == "bottom":
                y = height - image.size[1]
            else:
                raise ValueError(
                    "Invalid parameter: align(valign = %r)" % valign
                )

        copy.paste(image, (x, y), image if image.mode == "RGBA" else None)
        return copy

    return image




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			May 2009
"""
import re
import buffet
from cocktail.modeling import abstractmethod
from cocktail import schema
from woost.models.item import Item
from woost.models.emailtemplate import EmailTemplate

line_separator_expr = re.compile("[\r\n]+")


class TriggerResponse(Item):
    """A response action, to be executed when invoking the
    L{trigger<woost.models.Trigger>} it is bound to."""

    integral = True
    instantiable = False
    visible_from_root = False

    trigger = schema.Reference(
        type = "woost.models.Trigger",
        visible = False,
        bidirectional = True,
        integral = False
    )

    @abstractmethod
    def execute(self, items, user, batch = False, **context):
        """Executes the response with the supplied context.

        This method will be called when the response's trigger conditions are
        met. Subclasses of trigger response are expected to override this
        method in order to implement their particular logic.

        @param items: The list of items that received the condition that
            triggered the response.
        @type items: L{Item<woost.models.item.Item>} list

        @param user: The user that triggered the response.
        @type user: L{User<woost.models.user.User>}

        @param batch: Indicates if the response is being executed by a trigger
            that is set to operate in L{batch mode
            <woost.models.Trigger.batch>}.
        @type batch: bool

        @param context: Additional context. Available keys depend on the kind
            of action that triggered the response.
        """


class CustomTriggerResponse(TriggerResponse):
    """A trigger response that allows the execution of arbitrary python
    code."""
    instantiable = True

    code = schema.CodeBlock(
        language = "python",
        required = True
    )

    def execute(self, items, user, batch = False, **context):
        context.update(
            items = items,
            user = user,
            batch = batch
        )
        code = line_separator_expr.sub("\n", self.code)
        exec code in context


class SendEmailTriggerResponse(TriggerResponse):
    """A trigger response that allows to send an email."""

    instantiable = True

    email_template = schema.Reference(
        type = EmailTemplate,
        related_end = schema.Collection(),
        required = True
    )

    def execute(self, items, user, batch = False, **context):
        context.update(
            items = items,
            user = user,
            batch = batch
        )
        self.email_template.send(context)

# TODO: Implement other response types:
# NotifyUserTriggerResponse
# SendXMPPTriggerResponse (as an extension?)
# SendSMSTriggerResponse (as an extension?)
# WriteLogTriggerResponse (as an extension?)




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2009
"""
from tpv.currencies import currency_alpha
from cocktail.events import Event
from cocktail.translations import translations
from cocktail import schema
from woost.models import Item


class PaymentGateway(Item):

    instantiable = False
    visible_from_root = False

    transaction_notified = Event(
        """An event triggered when the payment gateway notifies the application
        of the outcome of a payment transaction.

        @param payment: The payment that the notification is sent for.
        @type payment: L{Payment<tpv.payment.Payment>}
        """
    )

    members_order = [
        "label",
        "test_mode",
        "currency"
    ]

    label = schema.String(
        required = True,
        translated = True
    )

    test_mode = schema.Boolean(
        required = True,
        default = True
    )

    currency = schema.String(
        required = True,
        enumeration = currency_alpha,
        text_search = False
    )

    def __translate__(self, language, **kwargs):
        return translations(self.__class__.name, language)




#-*- coding: utf-8 -*-
u"""Defines the `TwitterPublicationTarget`.

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from string import whitespace
from oauth2 import Client, Consumer, Token
from urllib import urlencode, urlopen
from simplejson import loads
from cocktail.events import when
from cocktail.iteration import first
from cocktail.translations import translations
from cocktail import schema
from woost.models import Item, Language
from woost.extensions.shorturls import ShortURLsExtension
from woost.extensions.shorturls.urlshortener import URLShortener
from woost.extensions.twitterpublication.exceptions import TwitterAPIError


class TwitterPublicationTarget(Item):

    visible_from_root = False

    tweet_max_length = 140

    members_order = [
        "title",
        "account",
        "app_id",
        "app_secret",
        "auth_token",
        "auth_secret",
        "languages"
    ]

    title = schema.String(
        required = True,
        unique = True,
        indexed = True,
        normalized_index = True,
        descriptive = True
    )

    account = schema.String(
        required = True
    )

    app_id = schema.String(
        required = True,
        text_search = False,
        listed_by_default = False
    )

    app_secret = schema.String(
        required = True,
        text_search = False,
        listed_by_default = False
    )

    auth_token = schema.String(
        editable = False,
        text_search = False,
        translate_value = lambda value, language = None, **kwargs:
            translations(
                "TwitterPublicationTarget.auth_token-%s"
                % ("conceded" if value else "pending"),
                language,
                **kwargs
            )
    )

    auth_secret = schema.String(
        visible = False
    )

    languages = schema.Collection(
        items = schema.String(
            enumeration = lambda ctx: Language.codes,
            translate_value = lambda value, language = None, **kwargs:
                "" if not value else translations(value)
        )
    )

    url_shortener = None

    def publish(self, publishable):

        if self.auth_token is None:
            raise ValueError(
                "Can't publish %s to %s: authorization token missing"
                % (publishable, self)
            )

        if self.auth_secret is None:
            raise ValueError(
                "Can't publish %s to %s: authorization secret missing"
                % (publishable, self)
            )

        consumer = Consumer(self.app_id, self.app_secret)
        token = Token(self.auth_token, self.auth_secret)
        client = Client(consumer, token)
        status = self.get_status(publishable)

        response, body = client.request(
            "https://api.twitter.com/1/statuses/update.json",
            "POST",
            urlencode({"status": status.encode("utf-8")})
        )

        if response["status"] != "200":
            raise TwitterAPIError(body)

    def find_post(self, publishable):

        query = "from:%s %s" % (self.account, self.get_uri(publishable))

        response = urlopen(
            "http://search.twitter.com/search.json?" + urlencode({
                "q": query,
                "result_type": "mixed"
            })
        )
        status = response.getcode()
        body = response.read()

        if status != 200:
            raise TwitterAPIError(body)

        posts = loads(body)["results"]
        return posts and posts[0] or None

    def get_status(self, publishable):
        uri = self.get_uri(publishable)
        title = self.get_title(publishable)
        title = self.trim(title, self.tweet_max_length - 1 - len(uri))
        return u"%s %s" % (title, uri)

    def get_title(self, publishable):
        return translations(publishable)

    def get_uri(self, publishable, *args, **kwargs):

        kwargs.setdefault("host", ".")
        url = publishable.get_uri(*args, **kwargs)

        if self.url_shortener:
            url = self.url_shortener.shorten_url(url)

        return url

    def trim(self, text, max_length, ellipsis = "..."):

        if len(text) > max_length:
            for i in range(max_length - len(ellipsis) - 1, -1, -1):
                print i, repr(text)
                if text[i] in whitespace:
                    text = text[:i].rstrip() + ellipsis
                    break

        return text


# If the 'shorturls' extension is enabled, add a member to define an URL
# shortening service for this target
@when(ShortURLsExtension.loading)
def add_url_shortener_reference(e):
    if e.source.enabled:
        TwitterPublicationTarget.add_member(
            schema.Reference("url_shortener",
                shadows_attribute = True,
                type = URLShortener,
                related_end = schema.Collection(),
                listed_by_default = False,

                # Default to the first defined service, if one exists
                default = schema.DynamicDefault(
                    lambda: first(URLShortener.select())
                )
            )
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from tpv.paypal import PayPalPaymentGateway as Implementation
from cocktail import schema
from cocktail.translations import translations
from cocktail.modeling import getter
from cocktail.controllers import context
from cocktail.controllers.location import Location
from woost.models import Document
from woost.extensions.payments.paymentgateway import PaymentGateway


class PayPalPaymentGateway(PaymentGateway, Implementation):

    members_order = [
        "business", "payment_successful_page", "payment_failed_page"
    ]

    default_label = schema.DynamicDefault(
        lambda: translations("PayPalPaymentGateway.label default")
    )

    instantiable = True

    business = schema.String(
        required = True,
        shadows_attribute = True,
        text_search = False
    )

    payment_successful_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    payment_failed_page = schema.Reference(
        type = Document,
        related_end = schema.Reference()
    )

    def _base_url(self):
        location = Location.get_current()
        location.relative = False
        location.path_info = context["cms"].application_uri()
        location.query_string = None

        return unicode(location)[:-1]

    @getter
    def notification_url(self):
        return self._base_url() + context["cms"].application_uri("payment_notification")

    @getter
    def payment_successful_url(self):

        if self.payment_successful_page:
            return self._base_url() \
                + context["cms"].uri(self.payment_successful_page) \
                + "?payment_id=%(payment_id)s"

        return None

    @getter
    def payment_failed_url(self):

        if self.payment_failed_page:
            return self._base_url() \
                + context["cms"].uri(self.payment_failed_page) \
                + "?payment_id=%(payment_id)s"

        return None

    def get_payment_form_data(self, payment_id, language = None):
        url, params = Implementation.get_payment_form_data(
            self, payment_id, language = language
        )

        payment = self.get_payment(payment_id)
        order = payment.order

        if order:
            if order.address:
                params.append(("address1", order.address[:100]))

            if order.town:
                params.append(("city", order.town[:40]))

            if order.country and order.country.code:
                params.append(("country", order.country.code))

            if order.postal_code:
                params.append(("zip", order.postal_code[:32]))

            if order.language:
                params.append(("lc", order.language.upper()))

        return url, params



#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			October 2008
"""
from cocktail.modeling import cached_getter
from cocktail import schema
from woost.controllers.backoffice.contentviews \
    import relation_content_views
from woost.controllers.backoffice.contentcontroller \
    import ContentController
from woost.controllers.backoffice.editcontroller import EditController
from woost.controllers.backoffice.useractions import get_user_action


class CollectionController(EditController, ContentController):

    view_class = "woost.views.BackOfficeCollectionView"

    def __init__(self, member):
        ContentController.__init__(self)
        EditController.__init__(self)
        self.member = member
        self.section = member.name

    @cached_getter
    def root_content_type(self):
        return self.member.items.type

    @cached_getter
    def action(self):
        return self.edited_item_action or self.collection_action

    @cached_getter
    def edited_item_action(self):
        return EditController.action(self)

    @cached_getter
    def collection_action(self):
        return self._get_user_action("collection_action")

    @cached_getter
    def action_content(self):
        if self.collection_action:
            return self.user_collection.selection
        else:
            return EditController.action_content(self)

    @cached_getter
    def user_collection(self):
        user_collection = ContentController.user_collection(self)
        user_collection.content_views_registry = relation_content_views
        user_collection.base_collection = \
            schema.get(self.stack_node.form_data, self.member)
        user_collection.default_order = None
        return user_collection

    @cached_getter
    def view_class(self):
        return self.member.edit_view or self.stack_node.item.collection_view

    @cached_getter
    def output(self):
        output = ContentController.output(self)
        output.update(EditController.output(self))
        output.update(
            member = self.member,
            selected_action = get_user_action("edit")
        )
        return output




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			April 2009
"""
from warnings import warn
from traceback import print_exc, format_exc
from threading import local
from weakref import WeakKeyDictionary
from cocktail import schema
from cocktail.translations import translations
from cocktail.events import when
from cocktail.persistence import datastore
from cocktail.controllers import UserCollection
from woost.models.action import Action
from woost.models.changesets import ChangeSet
from woost.models.site import Site
from woost.models.item import Item
from woost.models.role import Role
from woost.models.language import Language
from woost.models.usersession import get_current_user
from woost.models.messagestyles import (
    trigger_style,
    trigger_context_style,
    trigger_doesnt_match_style,
    trigger_match_style
)

_thread_data = local()

members_without_triggers = set([
    Item.changes,
    Item.last_update_time
])

verbose = False


def get_triggers_enabled():
    """Indicates if trigger activation is enabled for the current thread.
    @rtype: bool
    """
    return getattr(_thread_data, "enabled", True)

def set_triggers_enabled(enabled):
    """Enables or disables the activation of triggers for the current thread.

    @param enabled: Wether triggers should be enabled.
    @type enabled: bool
    """
    _thread_data.enabled = enabled


class Trigger(Item):
    """Describes an event."""

    instantiable = False
    visible_from_root = False
    members_order = [
        "title",
        "execution_point",
        "batch_execution",
        "matching_roles",
        "condition",
        "custom_context",
        "responses"
    ]

    title = schema.String(
        translated = True,
        descriptive = True
    )

    execution_point = schema.String(
        required = True,
        enumeration = ("before", "after"),
        default = "after",
        edit_control = "cocktail.html.RadioSelector",
        translate_value = lambda value, language = None, **kwargs:
            u"" if not value else translations(
                "woost.models.Trigger.execution_point " + value,
                language,
                **kwargs
            ),
        text_search = False
    )

    batch_execution = schema.Boolean(
        required = True
    )

    site = schema.Reference(
        type = "woost.models.Site",
        bidirectional = True,
        visible = False
    )

    responses = schema.Collection(
        items = "woost.models.TriggerResponse",
        bidirectional = True,
        related_key = "trigger"
    )

    matching_roles = schema.Collection(
        items = schema.Reference(
            type = Role,
            required = True
        ),
        related_end = schema.Collection(),
        edit_inline = True
    )

    condition = schema.CodeBlock(
        language = "python"
    )

    custom_context = schema.CodeBlock(
        language = "python"
    )

    def match(self, user, verbose = False, **context):

        # Check the user
        trigger_roles = self.matching_roles

        if trigger_roles:
            if user is None:
                print trigger_doesnt_match_style("user not specified")
                return False

            for role in user.iter_roles():
                if role in trigger_roles:
                    break
            else:
                print trigger_doesnt_match_style("user doesn't match")
                return False

        # Check the condition
        condition = self.condition

        if condition and not eval(condition, context):
            print trigger_doesnt_match_style("condition doesn't match")
            return False

        return True


class ContentTrigger(Trigger):
    """Base class for triggers based on content type instances."""

    edit_controller = \
        "woost.controllers.backoffice.triggerfieldscontroller." \
        "TriggerFieldsController"

    edit_view = "woost.views.TriggerFields"

    matching_items = schema.Mapping()

    def select_items(self, *args, **kwargs):
        user_collection = UserCollection(Item)
        user_collection.allow_paging = False
        user_collection.allow_member_selection = False
        user_collection.allow_language_selection = False
        user_collection.params.source = self.matching_items.get
        user_collection.available_languages = Language.codes
        return user_collection.subset

    def match(self, user, target = None, verbose = False, **context):

        # Check the target
        query = self.select_items()

        if not isinstance(target, query.type):
            if verbose:
                print trigger_doesnt_match_style("type doesn't match")
            return False
        else:
            for filter in query.filters:
                if not filter.eval(target):
                    if verbose:
                        print trigger_doesnt_match_style(
                            "filter doesn't match"
                        )
                    return False

        if not Trigger.match(
            self,
            user,
            target = target,
            verbose = verbose,
            **context
        ):
            return False

        return True


class CreateTrigger(ContentTrigger):
    """A trigger executed when an item is created."""
    instantiable = True


class InsertTrigger(ContentTrigger):
    """A trigger executed when an item is inserted."""
    instantiable = True


class ModifyTrigger(ContentTrigger):
    """A trigger executed when an item is modified."""

    instantiable = True
    members_order = ["matching_members", "matching_languages"]

    matching_members = schema.Collection(
        items = schema.String(required = True),
        edit_control = "woost.views.MemberList"
    )

    matching_languages = schema.Collection(
        items = schema.String(
            enumeration = lambda ctx: Language.codes,
            translate_value = lambda value, language = None, **kwargs:
                u"" if not value else translations(value, language, **kwargs)
        )
    )

    def match(self, user,
        target = None,
        member = None,
        language = None,
        verbose = False,
        **context):

        if self.matching_members:
            if member is None:
                if verbose:
                    print trigger_doesnt_match_style("member not specified")
                return False

            if (member.schema.full_name + "." + member.name) \
            not in self.matching_members:
                if verbose:
                    print trigger_doesnt_match_style("member doesn't match")
                return False

        if self.matching_languages:
            if language is None:
                if verbose:
                    print trigger_doesnt_match_style("language not specified")
                return False
            if language not in self.matching_languages:
                if verbose:
                    print trigger_doesnt_match_style("language doesn't match")
                return False

        if not ContentTrigger.match(
            self,
            user,
            target = target,
            member = member,
            language = language,
            verbose = verbose,
            **context
        ):
            return False

        return True


class DeleteTrigger(ContentTrigger):
    """A trigger executed when an item is deleted."""
    instantiable = True


class ConfirmDraftTrigger(ContentTrigger):
    """A trigger executed when a draft is confirmed."""
    instantiable = True


def trigger_responses(
    trigger_type,
    user = None,
    verbose = None,
    **context):

    target = context.get("target")

    if user is None:
        user = get_current_user()

    if verbose is None:
        verbose = globals()["verbose"]

    if get_triggers_enabled():

        # Create a structure to hold per-transaction data
        trans_dict = getattr(
            _thread_data,
            "trans_dict",
            None
        )

        if trans_dict is None:
            trans_dict = WeakKeyDictionary()
            _thread_data.trans_dict = trans_dict

        # Get the data for the current transaction
        trans = datastore.connection.transaction_manager.get()
        trans_data = trans_dict.get(trans)

        if trans_data is None:
            trans_triggers = {}
            modified_members = {}
            trans_data = {
                "triggers": trans_triggers,
                "modified_members": modified_members
            }
            trans_dict[trans] = trans_data
        else:
            trans_triggers = trans_data["triggers"]
            modified_members = trans_data["modified_members"]

        # Track modified members
        if issubclass(trigger_type, ModifyTrigger):
            target_modified_members = modified_members.get(target)

            if target_modified_members is None:
                target_modified_members = set()
                modified_members[target] = target_modified_members

            target_modified_members.add(
                (context["member"], context["language"])
            )

        if not Site.main:
            return None

        # Execute or schedule matching triggers
        for trigger in Site.main.triggers:

            if not isinstance(trigger, trigger_type):
                continue

            if verbose:
                print trigger_style(translations(trigger))
                print trigger_context_style("user", translations(user))
                for k, v in context.iteritems():
                    try:
                        v = translations(v) or unicode(v)
                    except:
                        pass
                    print trigger_context_style(k, v)

            if not trigger.match(
                user = user,
                verbose = verbose,
                **context
            ):
                continue

            print trigger_match_style("match")

            # Track modified / deleted items
            trigger_targets = trans_triggers.get(trigger)

            if trigger_targets is None:
                new_trans_trigger = True
                trigger_targets = set()
                trans_triggers[trigger] = trigger_targets
            else:
                new_trans_trigger = False

            if target is not None:
                trigger_targets.add(target)

            # Apply user defined customizations to the context
            # This can be specially useful to allow after-commit triggers to
            # capture state as it was when they were scheduled for execution
            # (ie. when deleting elements or confirming drafts).
            if trigger.custom_context:
                ctx = {
                    "self": trigger,
                    "context": context,
                    "user": user,
                    "target": target,
                    "trigger_targets": trigger_targets,
                    "modified_members": modified_members
                }
                exec trigger.custom_context in ctx
                context = ctx["context"]

            # Execute after the transaction is committed
            if trigger.execution_point == "after":

                if trigger.batch_execution:

                    # Schedule the trigger to be executed when the current
                    # transaction is successfully committed. Each batched
                    # trigger executes only once per transaction.
                    if new_trans_trigger:

                        def batched_response(successful, responses):
                            if successful:
                                try:
                                    for response in responses:
                                        response.execute(
                                            trigger_targets,
                                            user,
                                            batch = True,
                                            modified_members = modified_members,
                                            **context
                                        )

                                    datastore.commit()

                                except Exception, ex:
                                    warn("The following exception was raised "
                                         "by a deferred trigger response:")
                                    print_exc()
                                    datastore.abort()

                        trans.addAfterCommitHook(
                            batched_response,
                            (list(trigger.responses),)
                        )

                # Schedule the trigger to be executed when the current
                # transaction is successfully committed.
                else:
                    def delayed_response(successful, responses):
                        if successful:
                            try:
                                for response in responses:
                                    response.execute([target], user, **context)

                                datastore.commit()

                            except Exception, ex:
                                warn("The following exception was raised "
                                     "by a deferred trigger response:")
                                print_exc()
                                datastore.abort()

                    trans.addAfterCommitHook(
                        delayed_response,
                        (list(trigger.responses),)
                    )

            # Execute immediately, within the transaction
            else:
                for response in trigger.responses:
                    response.execute([target], user, **context)

@when(Item.instantiated)
def _trigger_instantiation_responses(event):
    trigger_responses(
        CreateTrigger,
        target = event.instance,
        values = event.values
    )

@when(Item.inserted)
def _trigger_insertion_responses(event):
    trigger_responses(
        InsertTrigger,
        target = event.source
    )

@when(Item.changed)
def _trigger_modification_responses(event):
    if event.source.is_inserted \
    and event.member not in members_without_triggers \
    and not isinstance(event.member, schema.Collection):
        trigger_responses(
            ModifyTrigger,
            target = event.source,
            member = event.member,
            language = event.language
        )

@when(Item.related)
@when(Item.unrelated)
def _trigger_relation_responses(event):
    if not isinstance(event.member, schema.Reference) \
    and event.member not in members_without_triggers:
        trigger_responses(
            ModifyTrigger,
            target = event.source,
            member = event.member,
            language = None
        )

@when(Item.deleted)
def _trigger_deletion_responses(event):
    trigger_responses(DeleteTrigger, target = event.source)

@when(Item.draft_confirmation)
def _trigger_draft_confirmation_responses(event):
    trigger_responses(ConfirmDraftTrigger, target = event.source)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Jordi Fernández <jordi.fernandez@whads.com>
"""
from cocktail import schema
from woost.extensions.blocks.block import Block


class ContainerBlock(Block):

    instantiable = True
    view_class = "cocktail.html.Element"

    blocks = schema.Collection(
        items = schema.Reference(type = Block),
        bidirectional = True,
        related_key = "containers",
        member_group = "content"
    )

    def descend_tree(self, include_self = False):

        if include_self:
            yield self

        for child in self.blocks:
            for descendant in child.descend_tree(include_self = True):
                yield descendant

    def create_view(self):

        view = Block.create_view(self)

        children_container = getattr(view, "block_content", view)

        for child in self.blocks:
            if child.enabled:
                child_view = child.create_view()
                children_container.append(child_view)

        return view




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from decimal import Decimal
from cocktail.translations import translations
from cocktail import schema
from woost.models import Item


class ShopOrderEntry(Item):

    listed_from_root = False

    members_order = [
        "shop_order",
        "product",
        "quantity"
    ]

    shop_order = schema.Reference(
        type = "woost.extensions.shop.shoporder.ShopOrder",
        bidirectional = True,
        required = True
    )

    product = schema.Reference(
        type = "woost.extensions.shop.product.Product",
        bidirectional = True,
        required = True
    )

    quantity = schema.Integer(
        required = True,
        min = 1
    )

    cost = schema.Decimal(
        required = True,
        default = Decimal("0"),
        editable = False
    )

    def __translate__(self, language, **kwargs):
        if self.draft_source is not None:
            return Item.__translate__(self, language, **kwargs)

        return "%s (%d)" % (
            translations(self.product, language),
            self.quantity
        )




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from woost.models.rendering import image_factory, content_renderers

@image_factory
def ecommerce_basket_thumbnail(item):
    return content_renderers.render(item, effects = "thumbnail(75,75)")




#-*- coding: utf-8 -*-
"""

@author:        Jordi Fernández
@contact:       jordi.fernandez@whads.com
@organization:  Whads/Accent SL
@since:         May 2010
"""
from cocktail import schema
from cocktail.modeling import cached_getter, getter
from cocktail.translations import translations
from cocktail.schema.expressions import CustomExpression
from cocktail.controllers.userfilter import (
    user_filters_registry,
    UserFilter
)
from woost.extensions.shop.shoporder import ShopOrder


class ShopOrderCostFilter(UserFilter):

    id = "shop_order_cost"
    operators = ["eq", "ne", "gt", "ge", "lt", "le"]

    @cached_getter
    def schema(self):
        return schema.Schema("ShopOrderCostFilter", members = [
            schema.String(
                "operator",
                required = True,
                default = "eq",
                enumeration = self.operators,
                text_search = False,
                translate_value = lambda value, language = None, **kwargs:
                    "" if not value
                    else translations(
                        "cocktail.html.UserFilterEntry operator " + value,
                        language,
                        **kwargs
                    )
            ),
            schema.Boolean(
                "include_shipping",
                default = False
            ),
            schema.Boolean(
                "include_taxes",
                default = False
            ),
            schema.Boolean(
                "include_discounts",
                default = False
            ),
            schema.Decimal(
                "value",
                required = True
            )
        ])

    @getter
    def expression(self):
        if self.operator == "eq":
            return CustomExpression(
                lambda item: item.calculate_cost(
                    include_shipping = self.include_shipping,
                    include_taxes = self.include_taxes,
                    include_discounts = self.include_discounts
                )["total"] == self.value
            )
        elif self.operator == "ne":
            return CustomExpression(
                lambda item: item.calculate_cost(
                    include_shipping = self.include_shipping,
                    include_taxes = self.include_taxes,
                    include_discounts = self.include_discounts
                )["total"] != self.value
            )
        elif self.operator == "gt":
            return CustomExpression(
                lambda item: item.calculate_cost(
                    include_shipping = self.include_shipping,
                    include_taxes = self.include_taxes,
                    include_discounts = self.include_discounts
                )["total"] > self.value
            )
        elif self.operator == "ge":
            return CustomExpression(
                lambda item: item.calculate_cost(
                    include_shipping = self.include_shipping,
                    include_taxes = self.include_taxes,
                    include_discounts = self.include_discounts
                )["total"] >= self.value
            )
        elif self.operator == "lt":
            return CustomExpression(
                lambda item: item.calculate_cost(
                    include_shipping = self.include_shipping,
                    include_taxes = self.include_taxes,
                    include_discounts = self.include_discounts
                )["total"] < self.value
            )
        else:
            return CustomExpression(
                lambda item: item.calculate_cost(
                    include_shipping = self.include_shipping,
                    include_taxes = self.include_taxes,
                    include_discounts = self.include_discounts
                )["total"] <= self.value
            )

user_filters_registry.add(ShopOrder, ShopOrderCostFilter)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
import re

html_expr = re.compile(r"</?[a-z][^>]*>")
li_expr = re.compile(r"<li(\s+[^>]*)?>")
br_expr = re.compile(r"<br\s*/?>")
block_end_expr = re.compile(r"</(p|li)>")
excessive_line_jumps_expr = re.compile(r"\n{2,}")

def export_content(content):
    if content:
        content = content.replace("\r", "\n")
        content = li_expr.sub("* ", content)
        content = br_expr.sub("\n", content)
        content = block_end_expr.sub("\n", content)
        content = html_expr.sub("", content)
        content = excessive_line_jumps_expr.sub("\n", content)
        content = content.strip("\n")
    return content




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			September 2009
"""
from cocktail.translations import translations

translations.define("Action sync_vimeo",
    ca = u"Sincronitzar amb Vimeo.com",
    es = u"Sincronizar con Vimeo.com",
    en = u"Synchronize with Vimeo.com"
)

translations.define("VimeoSynchronizationForm.vimeo_user_name",
    ca = u"Nom d'usuari de Vimeo",
    es = u"Nombre de usuario de Vimeo",
    en = u"Vimeo user name"
)

translations.define(
    "woost.extensions.vimeo.SynchronizationView synchronize button",
    ca = u"Sincronitzar",
    es = u"Sincronizar",
    en = u"Synchronize"
)

translations.define(
    "woost.extensions.vimeo.SynchronizationView outcome",
    ca = u"S'ha completat la sincronització amb el compte indicat.",
    es = u"Se ha completado la sincronización con la cuenta indicada.",
    en = u"Synchronization with the specified account complete."
)

translations.define(
    "woost.extensions.vimeo.SynchronizationView no changes",
    ca = u"No ha estat necessari realitzar cap actualització.",
    es = u"No ha sido necesario realizar ninguna actualización.",
    en = u"No update was needed."
)

translations.define(
    "woost.extensions.vimeo.SynchronizationView created videos title",
    ca = u"Vídeos afegits",
    es = u"Videos añadidos",
    en = u"Added videos"
)

translations.define(
    "woost.extensions.vimeo.SynchronizationView created videos explanation",
    ca = u"Els següents vídeos s'han incorporat al lloc web:",
    es = u"Los videos siguientes se han incorporado al sitio web:",
    en = u"The following videos were added to the web site:"
)

translations.define(
    "woost.extensions.vimeo.SynchronizationView modified videos title",
    ca = u"Vídeos modificats",
    es = u"Videos modificados",
    en = u"Modified videos"
)

translations.define(
    "woost.extensions.vimeo.SynchronizationView modified videos explanation",
    ca = u"S'ha actualitzat la informació dels vídeos següents:",
    es = u"Se ha actualizado la información de los videos siguientes:",
    en = u"The following videos had changes that have been reflected on the "
         u"web site:"
)

translations.define(
    "woost.extensions.vimeo.SynchronizationView deleted videos title",
    ca = u"Vídeos eliminats",
    es = u"Videos eliminados",
    en = u"Deleted videos"
)

translations.define(
    "woost.extensions.vimeo.SynchronizationView deleted videos explanation",
    ca = u"Els següents vídeos ja no són presents a vimeo.com i s'han "
         u"eliminat del lloc web:",
    es = u"Los videos siguientes ya no están disponibles en vimeo.com y se "
         u"han eliminado del sitio web:",
    en = u"The following videos are no longer available at vimeo.com and "
         u"have been deleted from the web site:"
)

translations.define("woost.extensions.vimeo.SynchronizationView cancel button",
    ca = u"Tancar",
    es = u"Cerrar",
    en = u"Close"
)

# VimeoVideo model
#------------------------------------------------------------------------------
translations.define("VimeoVideo",
    ca = u"Vídeo Vimeo",
    es = u"Video Vimeo",
    en = u"Vimeo video"
)

translations.define("VimeoVideo-plural",
    ca = u"Vídeos Vimeo",
    es = u"Videos Vimeo",
    en = u"Vimeo videos"
)

translations.define("VimeoVideo.content",
    ca = u"Contingut",
    es = u"Contenido",
    en = u"Content"
)

translations.define("VimeoVideo.video_file_data",
    ca = u"Dades del fitxer de vídeo",
    es = u"Datos del fichero de video",
    en = u"Video file data"
)

translations.define("VimeoVideo.stats",
    ca = u"Estadístiques",
    es = u"Estadísticas",
    en = u"Stats"
)

translations.define("VimeoVideo.vimeo_data",
    ca = u"Paràmetres d'allotjament",
    es = u"Parámetros de alojamiento",
    en = u"Hosting parameters"
)

translations.define("VimeoVideo.title",
    ca = u"Títol",
    es = u"Título",
    en = u"Title"
)

translations.define("VimeoVideo.description",
    ca = u"Descripció",
    es = u"Descripción",
    en = u"Description"
)

translations.define("VimeoVideo.vimeo_tags",
    ca = u"Etiquetes",
    es = u"Etiquetas",
    en = u"Tags"
)

translations.define("VimeoVideo.duration",
    ca = u"Duració",
    es = u"Duración",
    en = u"Duration"
)

translations.define("VimeoVideo.width",
    ca = u"Amplada",
    es = u"Ancho",
    en = u"Width"
)

translations.define("VimeoVideo.height",
    ca = u"Alçada",
    es = u"Alto",
    en = u"Height"
)

translations.define("VimeoVideo.approval_count",
    ca = u"Vots favorables",
    es = u"Votos favorables",
    en = u"Approval count"
)

translations.define("VimeoVideo.views_count",
    ca = u"Recompte de reproduccions",
    es = u"Recuento de reproducciones",
    en = u"Play count"
)

translations.define("VimeoVideo.comments_count",
    ca = u"Número de comentaris",
    es = u"Número de comentarios",
    en = u"Comments count"
)

translations.define("VimeoVideo.vimeo_video_id",
    ca = u"Identificador de vídeo de Vimeo",
    es = u"Identificador de video de Vimeo",
    en = u"Vimeo video ID"
)

translations.define("VimeoVideo.uri",
    ca = u"URL del vídeo",
    es = u"URL del video",
    en = u"Video URL"
)

translations.define("VimeoVideo.vimeo_user_name",
    ca = u"Nom d'usuari de Vimeo",
    es = u"Nombre de usuario de Vimeo",
    en = u"Vimeo user name"
)

translations.define("VimeoVideo.vimeo_user_url",
    ca = u"Adreça de l'usuari de Vimeo",
    es = u"Dirección del usuario de Vimeo",
    en = u"Vimeo user URL"
)

translations.define("VimeoVideo.upload_date",
    ca = u"Data de càrrega del vídeo a Vimeo",
    es = u"Fecha de carga del video en Vimeo",
    en = u"Upload date"
)




#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.modeling import OrderedDict
from cocktail.translations import translations
from cocktail import schema
from woost.models.rendering.contentrenderersregistry import (
    content_renderers,
    icon_renderers
)

image_factories = OrderedDict()

def image_factory(func):
    """A decorator used to register image factories."""
    image_factories[func.func_name] = func
    func.parameters = []
    return func


class ImageFactoryMember(schema.String):

    edit_control = "cocktail.html.DropdownSelector"

    def __init__(self, *args, **kwargs):
        kwargs.setdefault("enumeration", lambda ctx: image_factories.keys())
        schema.String.__init__(self, *args, **kwargs)

    def translate_value(self, value, language = None, **kwargs):
        if not value:
            return ""
        else:
            kwargs.setdefault("default", value)
            return translations(
                "woost.image_factory.%s" % value,
                language,
                **kwargs
            )


def parse_image_factory_parameters(parameters, string):
    """Parses additional parameters for an image factory."""

    if not string:
        return ()

    values = []
    chunks = string.split(".")
    chunk_count = len(chunks)

    for i, parameter in enumerate(parameters):

        if i >= chunk_count:
            chunk = parameter.produce_default()
        else:
            chunk = chunks[i]

        value = parameter.parse_request_value(None, chunk)

        for error in parameter.get_errors(value):
            raise error

        values.append(value)

    return tuple(values)

@image_factory
def default(item):
    """An image factory that returns the full size, unprocessed image for the
    item.
    """
    return content_renderers.render(item)

@image_factory
def icon16(item):
    """An image factory that returns the item's 16x16 icon."""
    return icon_renderers.render(item, size = 16)

@image_factory
def icon32(item):
    """An image factory that returns the item's 32x32 icon."""
    return icon_renderers.render(item, size = 32)

@image_factory
def backoffice_thumbnail(item):
    """Content preview or icon for thumbnails in the back office."""
    image = None
    if not isinstance(item, type):
        image = content_renderers.render(
            item, effects = "thumbnail(100,100)/frame(1,'ddd',4,'eee')"
        )
    return image or icon_renderers.render(item, size = 32)

@image_factory
def backoffice_small_thumbnail(item):
    """Smaller content preview or icon for thumbnails in the back office."""
    image = None
    if not isinstance(item, type):
        image = content_renderers.render(
            item, effects = "thumbnail(32,32)"
        )
    return image or icon_renderers.render(item, size = 16)

@image_factory
def image_gallery_close_up(item):
    """Image factory for the close up views in the `woost.views.ImageGallery`
    control.
    """
    return content_renderers.render(item, effects = "fit(900,700)")

@image_factory
def image_gallery_thumbnail(item):
    """Image factory for the thumbnails in the `woost.views.ImageGallery`
    control.
    """
    return content_renderers.render(item, effects = "fit(200,150)")




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
from cocktail.translations import translations
from cocktail.html import Element
from cocktail.html.checkbox import CheckBox
from woost.models import Item


class MemberList(Element):

    root_type = Item
    name = None
    value = ()

    def _build(self):
        self.add_resource("/cocktail/scripts/autocomplete.js")
        self.add_resource("/resources/scripts/MemberList.js")

    def _ready(self):

        if not self.name and self.data_display:
            self.name = self.data_display.get_member_name(
                self.member,
                self.language
            )

        if self.root_type:
            self.append(self.create_class_entry(self.root_type))

        Element._ready(self)

    def create_class_entry(self, cls):

        entry = Element()
        entry.add_class("content_type")

        entry.label = self.create_class_label(cls)
        entry.append(entry.label)

        entry.members_container = Element()
        entry.members_container.add_class("members")

        has_visible_members = False

        for member in cls.members(False).itervalues():
            if self.member_is_eligible(member):
                has_visible_members = True
                entry.members_container.append(
                    self.create_member_entry(member)
                )

        if has_visible_members:
            entry.append(entry.members_container)

        entry.derived_classes_container = Element()
        entry.derived_classes_container.add_class("derived_classes")
        entry.append(entry.derived_classes_container)

        for derived_schema in cls.derived_schemas(recursive = False):
            if derived_schema.visible:
                entry.derived_classes_container.append(
                    self.create_class_entry(derived_schema)
                )

        return entry

    def create_class_label(self, cls):
        label = Element("span")
        label.add_class("label")
        label.append(translations(cls.name))
        return label

    def member_is_eligible(self, member):
        return member.visible and member.name != "translations"

    def create_member_entry(self, member):

        value = member.schema.full_name + "." + member.name

        entry = Element()

        entry.check = CheckBox()
        entry.check["name"] = self.name
        entry.check.require_id()
        entry.check.value = value in self.value
        entry.check["value"] = value
        entry.append(entry.check)

        entry.label = Element("label")
        entry.label["for"] = entry.check["id"]
        entry.label.append(
            translations(member.schema.name + "." + member.name)
        )
        entry.append(entry.label)

        return entry




#-*- coding: utf-8 -*-
u"""

@author:        Jordi Fernández
@contact:       jordi.fernandez@whads.com
@organization:  Whads/Accent SL
@since:         March 2010
"""
from cocktail import schema
from woost.models import Document, Template, Controller
from woost.extensions.campaignmonitor.campaignmonitorlist import \
    CampaignMonitorList


class CampaignMonitorSubscriptionPage(Document):

    members_order = "lists", "subscription_form", "body"

    default_template = schema.DynamicDefault(
        lambda: Template.get_instance(
            qname = u"woost.extensions.campaignmonitor.subscription_template"
        )
    )

    default_controller = schema.DynamicDefault(
        lambda: Controller.get_instance(
            qname = u"woost.extensions.campaignmonitor.subscription_controller"
        )
    )

    body = schema.String(
        translated = True,
        listed_by_default = False,
        edit_control = "woost.views.RichTextEditor",
        member_group = "content"
    )

    lists = schema.Collection(
        items = schema.Reference(
            type = CampaignMonitorList
        ),
        edit_inline = True,
        member_group = "content",
        listed_by_default = False
    )

    subscription_form = schema.String(
        member_group = "content",
        default = \
            "woost.extensions.campaignmonitor.subscriptionform.SubscriptionForm",
        text_search = False
    )




#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			December 2008
"""
from urllib import urlencode
import cherrypy
from cocktail.modeling import getter
from cocktail.events import event_handler
from cocktail.controllers import Controller
from woost.models import Site


class BaseCMSController(Controller):
    """Base class for all CMS controllers."""

    def _render_template(self):

        # Pass the request context to the template
        cms = self.context["cms"]
        cms.producing_output(
            controller = self,
            output = self.output
        )

        return Controller._render_template(self)

    def application_uri(self, *args, **kwargs):
        """Builds an absolute URI from a set of path components and parameters.

        @param args: A set of path components, relative to the application
            root. Takes any object that can be expressed as an unicode string.

        @param kwargs: A set of query string parameters to be included on the
            generated URI.

        @return: The generated absolute URI.
        @rtype: unicode
        """
        path = (unicode(arg).strip("/") for arg in args)
        uri = (
            self.context["cms"].virtual_path
            + u"/".join(component for component in path if component)
        )

        if kwargs:
            uri += "?" + urlencode(
                dict(
                    (key, value)
                    for key, value in kwargs.iteritems()
                    if not value is None
                ),
                True
            )

        # Contextual URI prefix
        context_prefix = self.context.get("uri_prefix")

        if context_prefix:
            uri = context_prefix.rstrip("/") + "/" + uri.lstrip("/")

        if uri.startswith("./"):
            uri = uri[2:]

        return uri

    def contextual_uri(self, *args, **kwargs):
        """Builds an URI relative to the currently requested publishable item.

        @param args: A set of path components that will be appended to the
            publishable's URI. Accepts any object that can be expressed as an
            unicode string.

        @param kwargs: A set of query string parameters to be included on the
            generated URI.

        @return: The generated absolute URI.
        @rtype: unicode
        """
        publishable = self.context["publishable"]
        uri = Site.main.get_path(publishable)

        if uri is None:
            return None

        return self.application_uri(
            uri,
            *args,
            **kwargs
        )




#!/usr/bin/env python
#-*- coding: utf-8 -*-
u"""
Initializes the site's database.

*******************************************************************************
WARNING: this will clear all existing data, so use with care!
*******************************************************************************

The site's initialization is run automatically by the site installer, so
calling this script manually should seldomly be necessary. That said, it can be
useful during development (ie. to setup working copies from version control, or
to reset a site's state after some changes).
"""

# Load project settings
import _PROJECT_MODULE_

from woost.models.initialization import main

if __name__ == "__main__":
    main()




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			July 2009
"""
from cocktail.modeling import cached_getter, ListWrapper
from cocktail import schema
from cocktail.controllers import UserCollection
from cocktail.controllers.userfilter import GlobalSearchFilter
from woost.models import (
    ReadMemberPermission,
    get_current_user
)
from woost.controllers.backoffice.contentviews import global_content_views


class BackOfficeUserCollection(UserCollection):
    """An extended user collection that adds support for backoffice specific
    parameters.

    @var content_views_registry: A registry listing the different content views
        available to content types.
    @type: L{ContentViewsRegistry
             <woost.controllers.contentviews.ContentViewsRegistry>}
    """
    content_views_registry = global_content_views

    # Content view dependant capabilities
    #--------------------------------------------------------------------------
    @cached_getter
    def allow_member_selection(self):
        return self.content_view.allow_member_selection

    @cached_getter
    def allow_language_selection(self):
        return self.content_view.allow_language_selection

    @cached_getter
    def allow_filters(self):
        return self.content_view.allow_filters

    @cached_getter
    def allow_sorting(self):
        return self.content_view.allow_sorting

    @cached_getter
    def allow_grouping(self):
        return self.content_view.allow_grouping

    @cached_getter
    def allow_paging(self):
        return self.content_view.allow_paging

    # Type
    #--------------------------------------------------------------------------
    @cached_getter
    def type(self):
        type = UserCollection.type(self)

        if not type.visible:
            type = self.root_type

        return type

    @cached_getter
    def adapter(self):
        """The schema adapter used to produce data suitable for listing.
        @type: L{SchemaAdapter<cocktail.schema.adapter.SchemaAdapter>}
        """
        user = get_current_user()
        adapter = schema.Adapter()
        adapter.exclude([
            member.name
            for member in self.type.members().itervalues()
            if not member.visible
            or not user.has_permission(
                ReadMemberPermission,
                member = member
            )
        ])
        return adapter

    @cached_getter
    def schema(self):
        """The schema used by the produced listing of persistent items.
        @type: L{Schema<cocktail.schema.schema.Schema>}
        """
        content_schema = UserCollection.schema(self)

        if content_schema is not self.type:

            # Descriptive column
            content_schema.name = "BackOfficeContentView"
            content_schema.add_member(
                schema.Member(name = "element", searchable = False)
            )
            content_schema.members_order.insert(0, "element")

            # Type column
            if any(cls.visible for cls in self.type.derived_schemas()):
                content_schema.add_member(
                    schema.Member(name = "class", searchable = False)
                )
                content_schema.members_order.insert(1, "class")

        return content_schema

    # Content view
    #--------------------------------------------------------------------------
    @cached_getter
    def content_view(self):
        """The selected content view for the current request (flat view, tree
        view, thumbnails view, etc).
        @type: L{ContentView<woost.views.ContentView>}
        """
        content_view_type = None
        content_view_param = self.params.read(
            schema.String("content_view")
        )

        # Explicitly chosen content view
        if content_view_param is not None:
            for content_view_type in self.available_content_views:
                if content_view_type.content_view_id == content_view_param:
                    break
            else:
                content_view_type = None

        # Default content view
        if content_view_type is None:
            content_view_type = (
                self.content_views_registry.get_default(self.type)
                or self.available_content_views[0]
            )

        # Instantiate and initialize the content view
        content_view = content_view_type()

        params = self.content_views_registry.get_params(
            self.type,
            content_view_type
        )

        for key, value in params.iteritems():
            setattr(content_view, key, value)

        content_view._init_user_collection(self)
        return content_view

    @cached_getter
    def available_content_views(self):
        """The list of all content view classes available to the selected
        content type.
        @type: sequence of L{Element<cocktail.html.element.Element>} subclasses
        """
        return ListWrapper([
            content_view
            for content_view
                in self.content_views_registry.get(self.type)
            if content_view.compatible_with(self)
        ])

    # Members
    #--------------------------------------------------------------------------
    @cached_getter
    def default_members(self):
        content_schema = self.schema
        return set([member
                for member in UserCollection.default_members(self)
                if content_schema.get_member(member).listed_by_default])

    # Filters
    #--------------------------------------------------------------------------
    def should_ignore_filter(self, filter):
        return isinstance(filter, GlobalSearchFilter) and not filter.value

    # Tree expansion
    #--------------------------------------------------------------------------
    @cached_getter
    def expanded_items(self):
        if self.params.read(schema.String("expanded")) == "all":
            return "all"
        else:
            return self.params.read(
                schema.Collection(
                    "expanded",
                    type = set,
                    items = schema.Integer(),
                    default = set()
                )
            )




#-*- coding: utf-8 -*-
"""Provides functions for dealing with user notifications.

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from cocktail.controllers import session

def notify_user(message, category = None, transient = True):
    """Creates a new notification for the current user.

    Notifications are stored until a proper page is served to the user. It
    is up to the views to decide how these messages should be displayed.

    @param message: The message that will be shown to the user.
    @type message: unicode

    @param category: An optional free form string identifier that qualifies
        the message. Standard values include 'success' and 'error'.
    @type category: unicode

    @param transient: Indicates if the message should be hidden after a
        short lapse (True), or if it should remain visible until explicitly
        closed by the user (False).
    @type transient: bool
    """
    notifications = session.get("notifications")

    if notifications is None:
        session["notifications"] = notifications = []

    notifications.append((message, category, transient))

def pop_user_notifications(filter = None):
    """Retrieves pending notification messages that were stored through the
    L{notify_user} method.

    Retrieved messages are considered to be consumed, and therefore they
    are removed from the list of pending notifications.

    @param filter: If given, only those notifications matching the specified
        criteria will be retrieved. The criteria matches as follows:

            * If set to a string, it designates a single notification category
              to retrieve.
            * If set to a sequence (list, tuple, set), it designates a set of
              categories; notifications belonging to any of the specified
              categories will be retrieved.
            * If set to a callable, it will be used as filter function, taking
              a notification tuple as its single parameter, and returning True
              if the notification is to be retrieved, or False if it should be
              ignored.

    @return: The sequence of pending notification messages. Each message
        consists of a tuple with the message text, its category and wether
        or not it should be treated as a transient message.
    @rtype: sequence of (tuple of (unicode, unicode or None, bool))
    """
    notifications = session.get("notifications")

    if notifications is None:
        return []

    remaining = []

    if filter:
        matching = []

        if isinstance(filter, basestring):
            match = lambda notification: notification[1] == filter
        elif callable(filter):
            match = filter
        elif hasattr("__contains__", filter):
            match = lambda notification: notification[1] in filter
        else:
            raise TypeError(
                "The 'filter' parameter for the pop_user_notifications() "
                "function should be a string, a sequence or callable, got "
                "%s instead" % type(filter)
            )

        for notification in notifications:
            (matching if match(notification) else remaining).append(notification)

        notifications = matching

    session["notifications"] = remaining
    return notifications




#-*- coding: utf-8 -*-
"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			February 2010
"""
from cocktail.translations import translations
from cocktail import schema
from woost.models import Extension


translations.define("PDFExtension",
    ca = u"Exportació de documents PDF",
    es = u"Exportación de documentos PDF",
    en = u"PDF rendering"
)

translations.define("PDFExtension-plural",
    ca = u"Exportació de documents PDF",
    es = u"Exportación de documentos PDF",
    en = u"PDF rendering"
)

translations.define("PDFExtension.command",
    ca = u"Comanda",
    es = u"Comando",
    en = u"Command"
)


class PDFExtension(Extension):

    def __init__(self, **values):
        Extension.__init__(self, **values)
        self.extension_author = u"Whads/Accent SL"
        self.set("description",
            u"Permet publicar pàgines HTML en format PDF.",
            "ca"
        )
        self.set("description",
            u"Permite publicar páginas HTML en formato PDF.",
            "es"
        )
        self.set("description",
            u"Publishes HTML pages as PDF documents.",
            "en"
        )

    command = schema.String(
        required = True,
        default = u"python -m woost.extensions.pdf.renderpdf "
                  u"%(url)s %(output_file)s",
        text_search = False
    )

    def _load(self):

        import os
        from shutil import rmtree
        from tempfile import mkdtemp
        from subprocess import Popen
        import cherrypy
        from cocktail.controllers import Location, serve_file
        from woost.controllers import BaseCMSController

        extension = self

        def render_pdf(self):

            # Create a temporary folder to hold the PDF
            temp_path = mkdtemp()
            pdf_file_path = os.path.join(temp_path, "file.pdf")

            try:
                location = Location.get_current(relative = False)
                location.query_string["format"] = "html"

                # Create the file
                command = extension.command % {
                    "url": unicode(location),
                    "output_file": pdf_file_path
                }
                proc = Popen(command.encode("utf-8"), shell = True)
                stdout, stderr = proc.communicate()

                if proc.returncode:
                    raise OSError("Error generating PDF"
                        + (": " + stderr) if stderr else ""
                    )

                # Serve the file
                cherrypy.response.headers["Content-Type"] = "application/x-pdf"

                return serve_file(
                    pdf_file_path,
                    content_type = "application/x-pdf",
                    disposition = "attachment",
                    name = translations(self.context["publishable"]) + ".pdf"
                )

            finally:
                rmtree(temp_path)

        BaseCMSController.render_pdf = render_pdf
        BaseCMSController.allowed_rendering_formats = frozenset(
            list(BaseCMSController.allowed_rendering_formats)
          + ["pdf"]
        )

